@@ -1,5 +1,6 @@
+
 /*
-  Copyright (c) 2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2005-2007 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -39,9 +40,11 @@
 void
 commit_cb(void *ptr, const char *str)
 {
+  uim_agent_context *ua = (uim_agent_context *)ptr;
+
   debug_printf(DEBUG_NOTE, "commit_cb\n");
 
-  do_commit(str);
+  ua->comstr = add_commit_string(ua->comstr, str);
 }
 
 
@@ -91,7 +94,7 @@ candidate_activate_cb(void *ptr, int num, int limit)
   debug_printf(DEBUG_NOTE, 
 			   "candidate_activate_cb (num=%d,limit=%d)\n", num, limit);
 
-  new_candidate(ua->context, ua->pe->cand, num, limit);
+  new_candidate(ua->context, ua->cand, num, limit);
 }
 
 
@@ -103,7 +106,7 @@ candidate_select_cb(void *ptr, int index)
 
   debug_printf(DEBUG_NOTE, "candidate_select_cb (index: %d)\n", index);
 
-  ua->pe->cand->index = index;
+  ua->cand->index = index;
 }
 
 
@@ -114,7 +117,7 @@ candidate_shift_page_cb(void *ptr, int direction)
 
   debug_printf(DEBUG_NOTE, "candidate_shift_page_cb\n");
 
-  shift_candidate_page(ua->context, ua->pe->cand, direction);
+  shift_candidate_page(ua->context, ua->cand, direction);
 }
 
 
@@ -125,7 +128,7 @@ candidate_deactivate_cb(void *ptr)
 
   debug_printf(DEBUG_NOTE, "candidate_deactivate_cb\n");
 
-  clear_candidate(ua->pe->cand);
+  clear_candidate(ua->cand);
 }
 
 
@@ -136,31 +139,40 @@ prop_list_update_cb(void *ptr, const char *str)
 
   debug_printf(DEBUG_NOTE, "prop_list_update_cb\n");
 
-  update_prop_list(ua->prop, str);
-}
-
-
-void
-prop_label_update_cb(void *ptr , const char *str)
-{
-  uim_agent_context *ua = (uim_agent_context *)ptr;
-
-  debug_printf(DEBUG_NOTE, "prop_label_update_cb\n");
-
-  update_prop_label(ua->prop, str);
+  update_prop_list(ua->prop, ua->encoding, str);
 }
 
 
 void
 configuration_changed_cb(void *ptr)
 {
-
   uim_agent_context *ua = (uim_agent_context *)ptr;
 
   /* configuration of context has changed at uim side */
   debug_printf(DEBUG_NOTE, "configuration_changed_cb\n");
 
   update_context_configuration(ua);
-  
+}
+
+
+void
+switch_app_global_im_cb(void *ptr, const char *name)
+{
+  /* change default */
+  update_default_engine(name);
+
+  switch_context_im_all(name);
+}
+
+
+void
+switch_system_global_im_cb(void *ptr, const char *name)
+{
+  /* change default */
+  update_default_engine(name);
+
+  switch_context_im_all(name);
+
+  helper_send_im_change_whole_desktop(name);
 }
 
