@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2005-2007 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -40,23 +40,21 @@
 #include <string.h>
 #include <stdlib.h>
 
+#include <signal.h>
+#include <sys/types.h>
+
 #include <uim/uim.h>
-#include <uim/uim-helper.h>
 #include <uim/uim-im-switcher.h>
 
 #include "debug.h"
+#include "uim-el-types.h"
 
 #include "context.h"
 #include "encoding.h"
 #include "im.h"
-#include "helper-server.h"
+#include "helper-message.h"
 
-void helper_send_im_list(void);
-void helper_im_changed(char *request, char *engine_name);
-
-void helper_handler(void);
-
-int focused(uim_agent_context *ua);
-int unfocused(void);
+int helper_handler(uim_agent_context *ua, char *helper_message);
+void helper_send_im_change_whole_desktop(const char *name);
 
 #endif
