@@ -1,5 +1,5 @@
 ;; 
-;;  Copyright (c) 2005 uim Project http://uim.freedesktop.org/
+;;  Copyright (c) 2005-2007 uim Project http://uim.freedesktop.org/
 ;;
 ;;  All rights reserved.
 ;;
@@ -46,6 +46,7 @@
 
 (require 'uim-preedit)
 (require 'uim-candidate)
+(require 'uim-helper)
 
 
 ;;
@@ -92,7 +93,7 @@
 ;; Validation IM
 ;;
 (defun uim-check-im (im)
-  (if (assoc im (cdr uim-im-list))
+  (if (assoc im uim-im-alist)
       t
     (message (format "uim.el: invalid IM engine: %s" im))))
 
@@ -126,6 +127,8 @@
 ;;
 (defun uim-update-current-engine (engine)
 
+  (setq engine (car engine))
+
   (uim-debug (format "update-current-engine: %s" engine))
 
   (when (not (equal uim-current-im-engine engine))
@@ -133,28 +136,13 @@
     (run-hooks 'uim-update-current-engine-hook))
 
   ;; update current decoding code   
-  (let ((newcode (uim-get-emacs-code uim-current-im-engine)))
+  (let ((newcode (uim-get-emacs-encoding uim-current-im-engine)))
     (when (not (string= newcode uim-decoding-code))
       (setq uim-decoding-code newcode)
       (uim-change-process-encoding uim-decoding-code)))
   )
 
 
-;; Update mode-line string to "label1 label2".
-(defun uim-update-mode-line (label1 &optional label2)
-  (setq uim-current-prop-label (format "%s" (car label1)))
-  (if label2
-      (setq uim-current-prop-label 
-	    (concat uim-current-prop-label (format "%s" (car label2)))))
-
-  (setq uim-mode-line-string 
-	(format " U %s[%s]"
-		uim-current-im-engine uim-current-prop-label))
-
-  )
-
-
-
 ;;	
 ;; Create new context
 ;;
@@ -166,7 +154,7 @@
     ;;    Default context's encoding is determined by 
     ;;  the name of default IM 
     (uim-do-send-recv-cmd 
-     (format "%d NEW %s" id (uim-get-uim-code (uim-get-default-engine))))
+     (format "%d NEW %s" id (uim-get-uim-encoding (uim-get-default-engine))))
     id))
 
 
@@ -200,18 +188,18 @@
 ;;
 (defun uim-focused ()
   (uim-change-process-encoding uim-decoding-code)
-  (uim-do-send-recv-cmd (format "%d FOCUSED" uim-context-id))
   (setq uim-focused-buffer (current-buffer))
+  (uim-do-send-recv-cmd (format "%d FOCUSED" uim-context-id))
   )
 
 ;;
 ;; Unfocused
 ;;
 (defun uim-unfocused ()
+  (setq uim-focused-buffer nil)
   ;; don't send a message to uim-el-agent if it has been dead
   (if uim-el-agent-process
       (uim-do-send-recv-cmd (format "%d UNFOCUSED" uim-context-id)))
-  (setq uim-focused-buffer nil)
   )
 
 
@@ -247,32 +235,23 @@
     (setq uim-recent-buffer (current-buffer))
     
     ;; remove Uim focus from previous buffer
-    (if (and (bufferp uim-focused-buffer)
+    (when (and (bufferp uim-focused-buffer)
 	     (buffer-name uim-focused-buffer))
 	(save-current-buffer
 	  (set-buffer uim-focused-buffer)
-	  (when (and uim-initialized
-		     uim-mode)
-	    (uim-debug (format "unfocused %s" 
-			       (buffer-name (current-buffer))))
+	(when (and uim-initialized uim-mode)
+	  (uim-debug (format "unfocused %s" (buffer-name (current-buffer))))
 	    (uim-unfocused)
 	    )))
 	    
-	      
     ;; set Uim focus to current buffer if it has Uim context
-    (if (and uim-initialized
-	     uim-mode)
-	(progn 
-	  (uim-debug (format "focused %s" 
-			     (buffer-name (current-buffer))))
-
+    (when (and uim-initialized uim-mode)
+      (uim-debug (format "focused %s" (buffer-name (current-buffer))))
 	  (uim-focused)
 	  )
-      ;; no Uim context in current buffer
-      )
-
-    (uim-debug (format "current-buffer %s / uim-focused-buffer %s" (current-buffer) uim-focused-buffer ))
 
+    (uim-debug (format "current-buffer %s / uim-focused-buffer %s" 
+		       (current-buffer) uim-focused-buffer ))
     )
   )
 
@@ -301,37 +280,34 @@
   (set-process-coding-system uim-el-agent-process outcode 'iso-8859-1)
   )
 
+;; 
+;; Get IM information from uim-im-alist
 ;;
-;; Get IM information from uim-im-list
-;;
-(defun uim-get-im-info (im info)
-  (cdr (assoc info (cdr (assoc im (cdr uim-im-list))))))
-
-(defun uim-get-uim-code (im)
-  (or (uim-get-im-info im 'uim-code)
-      "ISO-8859-1"))
-
-(defun uim-get-emacs-code (im)
-  (or (uim-get-im-info im 'emacs-code)
-      'iso-8859-1))
+(defun uim-get-im-info (im field)
+  (nth field (or (assoc im uim-im-alist)
+		 (assoc "direct" uim-im-alist))))
 
 (defun uim-get-uim-lang (im)
-  (or (uim-get-im-info im 'uim-lang)
-      "Unknown"))
+  (uim-get-im-info im 1))
 
 (defun uim-get-emacs-lang (im)
-  (or (uim-get-im-info im 'emacs-lang)
-      "ASCII"))
+  (uim-get-im-info im 2))
+
+(defun uim-get-emacs-encoding (im)
+  (uim-get-im-info im 3))
+
+(defun uim-get-uim-encoding (im)
+  (uim-get-im-info im 4))
+
 
 ;;
 ;; Change IM engine.
-;;   im should be valid egine name.
 ;; 
 (defun uim-change-im (im)
   ;; change decoding method temporarily to receive encoded IM label etc.
-  (uim-change-process-encoding (uim-get-emacs-code im))
+  (uim-change-process-encoding (uim-get-emacs-encoding im))
   (uim-do-send-recv-cmd 
-   (format "%d CHANGE %s %s" uim-context-id im (uim-get-uim-code im)))
+   (format "%d CHANGE %s" uim-context-id im))
 
   (if uim-default-im-prop
       (uim-prop-activate uim-default-im-prop)))
@@ -340,10 +316,9 @@
 ;;
 ;; Set each IM's output encoding 
 ;;
-(defun uim-set-encoding (im)
-  (let ((uim-code (uim-get-uim-code im)))
-    (uim-do-send-recv-cmd 
-     (format "0 SETENC %s %s" im uim-code))))
+(defun uim-set-encoding (im encoding)
+  (uim-do-send-recv-cmd 
+   (format "0 SETENC %s %s" im encoding)))
 
 
 
@@ -386,8 +361,13 @@
   )
 
 
-(defun uim-update-keybind ()
-  (uim-copy-toggle-key 'uim-mode))
+(defun uim-im-init ()
+  (when (not uim-im-initialized)
+    (uim-init-im-alist)
+    (uim-init-im-encoding)
+    (setq uim-im-initialized t)
+    ))
+
 
 ;;
 ;; Activate uim
@@ -397,7 +377,11 @@
   (uim-debug "uim-mode-on")
 
   (if uim-mode
-      (message "uim.el: uim-mode is already activated. (buffer %s)" (current-buffer)))
+      (message "uim.el: uim-mode is already activated. (buffer %s)" 
+	       (current-buffer)))
+
+  ;; Initialize IM encoding
+  (uim-im-init)
 
   (if (not buffer-read-only)
       (progn 
@@ -408,7 +392,6 @@
 
 	;; enable and update keymap
 	(uim-enable-mode-keymap)
-	(uim-update-keybind)
 
 	;; focus to current context
 	(uim-focused)
@@ -435,6 +418,8 @@
 
   (uim-unfocused)
 
+  (uim-process-agent-output '(("e")))
+
   ;; update mode-line
   (force-mode-line-update)
 
@@ -470,6 +455,8 @@
 ;;
 (defun uim-change-major-mode ()
 
+  (uim-debug "uim-change-major-mode")
+
   (when uim-initialized
        (uim-unfocused)
        (uim-delete-context))
@@ -483,6 +470,9 @@
   (let (proc
 	(buffer (get-buffer-create uim-el-agent-buffer-name)) )
 
+    (if (not uim-el-helper-agent-process)
+	(setq uim-el-helper-agent-process (uim-el-helper-agent-start)))
+
     ;; erase buffer
     (save-current-buffer
       (set-buffer buffer) (erase-buffer))
@@ -517,6 +507,9 @@
     (setq uim-el-agent-buffer buffer)
 
     (set-process-sentinel proc 'uim-process-sentinel)
+
+    (message "uim.el: starting uim-el-agent... done")
+
     proc 
     ))
 
@@ -558,24 +551,14 @@
 ;; Update IM list
 ;;
 (defun uim-update-imlist (imlist)
-  ;; imlist: default (engine lang_code Language explanation encoding)
-
-  (let ((default (car imlist)))
-
-    ;; initialize default IM engine name with the context's default value
-    (when (not uim-default-im-engine)
-      (uim-update-default-engine default))
-
-
-    (setq 
-     uim-im-list 
-     (cons default
-	   (mapcar (lambda (x) 
-		     (cons (car x)
-			   (cdr (or (assoc (nth 2 x) uim-lang-code-alist)
-				    (assoc "Other" uim-lang-code-alist)))))
-		   (cdr imlist))))
-    )
+  (setq uim-im-alist 
+	(mapcar '(lambda (x) 
+		   (let ((im (nth 0 x))
+			 (lang-uim (nth 2 x)))
+		     (cons im
+			   (or (assoc lang-uim uim-lang-code-alist)
+			       (assoc "Other" uim-lang-code-alist)))))
+		imlist))
   )
 
 
@@ -583,30 +566,29 @@
 ;; Update property label
 ;;
 (defun uim-update-label (label)
-  ;; label: ( "current" ("label_abbr"  "label") ... )
-  (let (label1 label2 mode-str)
-    ;; current IM engine name
-    (uim-update-current-engine (car label))
-
-    (setq label (cdr label))
-
-    (setq label1 (car label))
-    (setq label (cdr label))
-
-    (when label
-      (setq label2 (car label)))
-
-    ;; Update mode-line string to "label1 label2".
-    (setq mode-str (format "%s" (car label1)))
-
-    (if label2 
-	(setq mode-str (concat mode-str (format "%s" (car label2)))))
-
-    (setq uim-mode-line-string 
-	  (format " U %s[%s]" uim-current-im-engine mode-str))
+  (let ((mode-str ""))
+    (mapcar
+     '(lambda (x)
+	(cond ((string= (nth 0 x) "im-mode")
+	       (setq mode-str (concat mode-str (nth 2 x))))
+	      ((string= (nth 0 x) "im-name")
+	       (setq uim-im-indication-id (nth 1 x))
+	       (setq uim-im-name-str (nth 3 x)))
+	      )
+	)
+     label)
 
+    (setq uim-im-mode-str mode-str)
 
+    (setq uim-mode-line-string
+	  (concat (if (or uim-show-im-name uim-show-im-mode) " ")
+		  (if uim-show-im-name
+		      uim-im-name-str "")
+		  (if uim-show-im-mode
+		      (format "[%s]" uim-im-mode-str) "")))
     )
+
+  (run-hooks 'uim-update-label-hook)  
   )
 
 
@@ -648,6 +630,15 @@
 	(uim-do-send-recv-cmd (format "%d SHOW" uim-context-id))))))
 
 
+;;
+;; Reset all uim.el buffer local variables
+;;
+(defun uim-init-all-local-var ()
+   (mapcar
+    '(lambda (x)
+       (set (car x) (cdr x)))
+    uim-local-var))
+
 ;; 
 ;; Sentinel of uim-el-agent 
 ;;   STOP state is not supported yet
@@ -667,7 +658,8 @@
 	  ;; disable uim
 	  (uim-force-off)
 	  ;; reset all local variables
-	  (kill-all-local-variables)
+	  (uim-init-all-local-var)
+	  (setq uim-im-initialized nil)
 	  ))
       (setq bufs (cdr bufs)))
     ))
@@ -678,7 +670,7 @@
 ;; 
 (defun uim-send-cmd (cmd serial)
 
-  (uim-debug (concat (format "%s " serial) cmd ))
+  (uim-debug (concat (format "%s " serial) cmd " (" (current-time-string) ")"))
 
   ;; check process status and start if it's not running
   (if (not uim-el-agent-process)
@@ -734,9 +726,14 @@
 ;; Wait reply from uim-el-agent
 ;;   Check serial number 
 ;;
-(defun uim-wait-recv(serial)
+(defun uim-wait-recv (serial)
 
-  (let (rstr)
+  (let (rstr
+	(helper-filter (process-filter uim-el-helper-agent-process)))
+
+    (set-process-filter uim-el-helper-agent-process nil)
+
+    (uim-debug "wait recv")
 
     (let ((start (nth 1 (current-time)))
 	  (accept uim-el-agent-accept-timeout))
@@ -754,6 +751,16 @@
 	  ;; double the accepting time if waiting again
 	  (setq accept (* accept 2)))))
 
+    (save-current-buffer
+      (set-buffer uim-el-helper-agent-buffer)
+      (when (> (buffer-size) 0)
+	(setq uim-helper-message 
+	      (concat uim-helper-message
+		      (buffer-substring 1 (point-max))))
+	(erase-buffer)))
+    (set-process-filter uim-el-helper-agent-process helper-filter)
+
+    (unwind-protect
     (cond ((not (process-status uim-el-agent-process))
 	   ;; uim-el-agent is dead
 	   (message "uim.el: state of uim-el-agent has been changed (died?)"))
@@ -768,17 +775,23 @@
 	   ;;   rstr: ( serial# context-id (...) )
 	   (uim-process-agent-output (cddr (read rstr)))
 	   )
-	  )))
+	      )
+      (uim-helper-message-processor)
+      )
+    )
+  )
 
 
 ;;
 ;; Send message to uim-el-agent and wait return
 ;;
 (defun uim-do-send-recv-cmd (cmd)
+  (setq uim-last-cmd cmd) ;; keep last command
   (let ((serial (uim-get-communication-serial-number)))
     (if (uim-send-cmd cmd serial)
 	(uim-wait-recv serial))))
 
+
 ;;
 ;; Save face property
 ;;
@@ -855,6 +868,26 @@
 		     'face (car facelist)))
 
 
+;;
+;; Update font 
+;;
+(defun uim-update-font ()
+  (mapcar 
+   '(lambda (x)
+      (let ((font (or (cdr (assq 'font (frame-parameters)))
+		      (face-font 'default))))
+	(set-face-font x font)))
+		     
+   '(uim-preedit-face 
+     uim-preedit-underline-face 
+     uim-preedit-highlight-face
+     uim-preedit-highlight-underline-face
+     uim-separator-face
+     uim-candidate-odd-face
+     uim-candidate-even-face
+     uim-candidate-selected-face
+     uim-candidate-nth-face)))
+
 ;;
 ;; Lock appearance of buffer
 ;;
@@ -885,8 +918,6 @@
 
  
 
-
-
 ;;
 ;; Process inputted key
 ;;
@@ -902,9 +933,21 @@
       (setq uim-show-keystrokes nil))
 
   ;; send an input key code to uim-el-agent
-  (let ((keyvec (uim-this-command-keys))
+
+  (if current-prefix-arg
+      (setq uim-prefix-arg current-prefix-arg))
+
+  
+  (if uim-prefix-arg
+      (uim-debug (format "uim-prefix-arg %s" uim-prefix-arg)))
+
+
+  (let ((keyvec (uim-this-command-keys current-prefix-arg))
 	sendkey newvec bypass count mouse)
 
+    (if uim-prefix-arg
+	(setq current-prefix-arg uim-prefix-arg))
+
     (if keyvec
 	(progn
 	  (if uim-show-keystrokes
@@ -925,9 +968,11 @@
 			(setq bypass t))
 		       
 		       (current-prefix-arg
-			(uim-debug "with prefix arg")
 			(setq bypass t)
-			(setq count (prefix-numeric-value arg)))
+			;;(setq count (prefix-numeric-value arg))
+			(setq count (prefix-numeric-value current-prefix-arg))
+			(uim-debug (format "with prefix arg: count is %s" count))
+			)
 		       
 		       ((and (not (or uim-preedit-displayed 
 				      uim-candidate-displayed))
@@ -947,10 +992,12 @@
 		 (setq sendkey keyvec)
 		 
 		 (cond (current-prefix-arg
-			;; (uim-debug (format "with prefix arg %s"
-			;; current-prefix-arg))
 			(setq bypass t)
-			(setq count (prefix-numeric-value arg)))
+			(setq count (prefix-numeric-value arg))
+			(setq count (prefix-numeric-value current-prefix-arg))
+			(uim-debug (format "with prefix arg: count is %s"
+					   count))
+			)
 		       ((and (eventp event)
 			     (memq (event-basic-type event) 
 				   '(mouse-1 mouse-2 mouse-3 mouse-4 mouse-5)))
@@ -982,25 +1029,62 @@
 				  (eq (aref sendkey 0) 'escape)))))
 	    (uim-debug "bypass")
 	    (setq bypass t))
-		 
 
+
+	  (setq uim-retry-keys nil)
+
+	  (unwind-protect
+	      (if (and uim-preedit-keymap-enabled
+		       (or (eq (car-safe (aref sendkey 0)) 'menu-bar)
+			   (eq (car-safe (aref sendkey 0)) 'tool-bar)))
+		  (uim-do-send-recv-cmd (format "%d NOP" uim-context-id))
 	  (if (not bypass)
 	      (uim-do-send-recv-cmd (format "%d %s" uim-context-id sendkey))
 	    (if mouse
 		(uim-process-mouse-event event)
-	      (uim-process-keyvec uim-last-key-vector count)))
+	      (uim-process-keyvec uim-last-key-vector count)
+
+	      ;; display "ESC ESC ESC" or something
+
+	      (if uim-show-keystrokes
+		  (let (message-log-max keymsg)
+		    (setq keymsg (if uim-prefix-arg-vector
+				     (concat (key-description uim-prefix-arg-vector)
+					     " ")
+				   nil))
+		    (mapcar '(lambda (x)
+			       (setq keymsg (concat keymsg
+						(key-description (vector x))
+						" ")))
+			    (append uim-last-key-vector nil))
+		    (message keymsg))
+		)
+		    )))
+	  
+	    (progn
+	  (uim-debug "reset prefix variables")
+	  (setq uim-prefix-arg nil)
+	  (setq uim-prefix-arg-vector nil)
+	  (setq uim-prefix-ignore-next nil)
+	      )
+	    )
+
 	  )
-      
+
+      ;; if keyvec is nil
+      (setq uim-deactivate-mark nil)
+
       (if (not uim-show-keystrokes)
 	  (if (sit-for echo-keystrokes)
 	      (setq uim-show-keystrokes t)))
 
+      ;; display "ESC-" or something
       (if uim-show-keystrokes
 	  (let (message-log-max)
-	    (message (concat (key-description uim-stacked-key-vector) "-"))))
-
-      )
-    )
+	    (message (concat (key-description (vconcat uim-prefix-arg-vector
+						       uim-stacked-key-vector))
+			     "-"))))
+      ))
 
   (if uim-emacs
       (setq deactivate-mark uim-deactivate-mark))
@@ -1009,179 +1093,274 @@
 
 
 
+(defun uim-enter-preedit-mode ()
+  ;; change keymap and freeze faces at first time
+  (uim-enable-preedit-keymap)
+  (when (= (minibuffer-depth) 0)
+    (uim-freeze-buffer)
+    (setq uim-buffer-read-only buffer-read-only)
+    (setq buffer-read-only t))
+  )
+
+(defun uim-leave-preedit-mode ()
+  (uim-disable-preedit-keymap)
+  (uim-unfreeze-buffer)
+  (setq buffer-read-only uim-buffer-read-only)
+  )
+
+
 
 ;; process return expression from uim-el-agent 
 (defun uim-process-agent-output (str)
-  (let (preedit-existed
-	candidate-existed
-	key commit preedit candidate default label imlist)
+  (catch 'process-agent-output
+    (let (;;(inhibit-quit t)
+	  preedit-existed
+	  candidate-existed
+	  key commit preedit candidate default im label imlist helpermsg
+	  )
+      
+      (uim-debug (format "%s" str))
+      
 
-    (let ((modified (buffer-modified-p)))
-      ;; remove candidate anyway
-      (when uim-candidate-displayed
-	(setq candidate-existed t)
-	(uim-remove-candidate)
-	(setq uim-candidate-displayed nil)
-	(goto-char uim-candidate-start))
+      (let ((modified (buffer-modified-p)))
 
-      ;; remove preedit anyway
-      (when uim-preedit-displayed
-	(setq preedit-existed t)
-	(uim-remove-preedit)
-	(setq uim-preedit-displayed nil)
-	(goto-char uim-preedit-start))
+	;; remove candidate anyway
+	(when uim-candidate-displayed
+	  (setq candidate-existed t)
+	  (let ((inhibit-read-only t))
+	    (uim-remove-candidate))
+	  (setq uim-candidate-displayed nil)
+	  (uim-goto-char uim-preedit-start))
 
-      ;; restore modified flag
-      (set-buffer-modified-p modified))
+	;; remove preedit anyway
+	(when uim-preedit-displayed
+	  (setq preedit-existed t)
+	  (let ((inhibit-read-only t))
+	    (uim-remove-preedit))
+	  (setq uim-preedit-displayed nil)
+	  (uim-goto-char uim-original-cursor))
 
-    (uim-debug (format "%s" str))
+	;; restore cursor point
+	(when (and uim-preedit-keymap-enabled uim-original-cursor)
+	  (goto-char uim-original-cursor))
 
+	;; restore modified flag
+	(set-buffer-modified-p modified))
 
-    (mapcar 
-     '(lambda (x) 
-	(let ((rcode (car x))
-	      (rval (cdr x)))
+      (mapcar 
+       '(lambda (x) 
+	  (let ((rcode (car x))
+		(rval (cdr x)))
 	
-	  (cond ((string= rcode "n") ;; uim returns key code
-		 (if uim-last-key-vector
-		     (progn
-		       (setq key uim-last-key-vector)
-		       (uim-debug (format "last-key: %s" key)))
-		   (setq key (car rval)))
-		 )
-		((string= rcode "s") ;; commit string
-		 (setq commit (append commit (list (car rval))))
-		 )
-		((string= rcode "c") ;; candidate data
-		 (setq candidate rval)
-		 )
-		((string= rcode "p") ;; preedit
-		 (setq preedit rval)
-		 )
-		((string= rcode "d") ;; default engine
-		 (setq default (car rval))
-		 )
-		((string= rcode "l") ;; label
-		 (setq label rval)
-		 )
-		((string= rcode "L") ;; IM list
-		 (setq imlist rval)
-		 )
-		((string= rcode "e") ;; IM list
-		 )
-		)
-	  )) 
-     str)
+	    (cond ((string= rcode "n") ;; uim returns key code
+		   (if uim-last-key-vector
+		       (progn
+			 (setq key uim-last-key-vector)
+			 (uim-debug (format "last-key: %s" key)))
+		     (setq key (car rval)))
+		   )
+		  ((string= rcode "s") ;; commit string
+		   (setq commit (append commit (list (car rval))))
+		   )
+		  ((string= rcode "c") ;; candidate data
+		   (setq candidate rval)
+		   )
+		  ((string= rcode "p") ;; preedit
+		   (setq preedit rval)
+		   )
+		  ((string= rcode "d") ;; default engine
+		   (setq default (car rval))
+		   )
+		  ((string= rcode "i") ;; current im
+		   (setq im rval)
+		   )
+		  ((string= rcode "l") ;; label
+		   (setq label rval)
+		   )
+		  ((string= rcode "h") ;; helper message
+		   (setq helpermsg (append helpermsg rval))
+		   )
+		  ((string= rcode "L") ;; IM list
+		   (setq imlist rval)
+		   )
+		  )
+	    )) 
+       str)
+
+      (when helpermsg
+	(uim-helper-send-message helpermsg))
+
+      (when default
+	(uim-update-default-engine default))
+
+      (when im
+	(uim-update-current-engine im))
+
+      (when label
+	(uim-update-label label))
+
+      (when imlist
+	(uim-update-imlist imlist))
 
 
-    (when default
-      (uim-update-default-engine default))
-
-    (when label
-      (uim-update-label label))
-
-    (when imlist
-      (uim-update-imlist imlist))
-
-
-    (if commit
-	(let ((buffer-undo-list-saved uim-buffer-undo-list-saved))
-	  ;; restore undo history before committing
-	  (when buffer-undo-list-saved
-	    (uim-debug "call restore undo")
-	    (uim-restore-undo))
+      (if commit
 
 	  ;; insert committed strings
-	  (mapcar
-	   '(lambda (x) 
-	      (insert x)
-	      (uim-debug (format "insert %s" x))
-	      (undo-boundary))
-	   commit)
+	  (let ((inhibit-read-only t)
+		(buffer-frozen uim-buffer-frozen))
+	    (if uim-buffer-frozen
+		(uim-unfreeze-buffer))
 
-	  ;; save undo hisotry again
-	  (when buffer-undo-list-saved
-	    (uim-debug "call save undo")
-	    (uim-save-undo)))
-      )
+	    (mapcar
+	     '(lambda (x)
+		(let ((start (point)))
+		  
+		  ;; enable buffer-undo temporarily
+		  (when uim-buffer-undo-list-saved
+		    (setq buffer-undo-list nil)
+		    (buffer-enable-undo))
 
+		  (insert x)
+		  (uim-debug (format "insert %s" x))
 
-    (if (or preedit candidate)
-	;; process preedit/candidate
-	(let ((modified (buffer-modified-p)))
+		  ;; disable buffer-undo temporarily
+		  (when uim-buffer-undo-list-saved
+		    (let (delatom)
+		      (defun delatom (x)
+			(if (and x (not (atom x)))
+			    (if (and (car x) (atom (car x)))
+				(delatom (cdr x))
+			      (cons (car x) (delatom (cdr x))))
+			  x))
+		      (setq buffer-undo-list (delatom buffer-undo-list))
+		      )
 
-	  ;; save undo list if not saved
-	  (when (not uim-buffer-undo-list-saved)
-	    (uim-flush-concat-undo)
-	    (uim-debug "call save undo")
-	    (uim-save-undo))
+		    (setq uim-buffer-undo-list
+			  (append buffer-undo-list uim-buffer-undo-list))
+		    (setq buffer-undo-list nil)
+		    (buffer-disable-undo))
+		  )
+		)
+	     commit)
 
-	  ;; change keymap and freeze faces at first time
-	  (when (not uim-preedit-keymap-enabled)
-	    (uim-debug "call enable keymap")
-	    (uim-enable-preedit-keymap)
-	    (if (= (minibuffer-depth) 0)
-		(uim-freeze-buffer)))
+	    (if auto-fill-function
+		(funcall auto-fill-function))
 
-	  ;; show preedit
-	  (if preedit
-	      (progn
-		(setq uim-preedit-displayed t)
-		(uim-insert-preedit preedit))
-	    (setq uim-candidate-start (point))
-	    (setq uim-candidate-vofs 0)
-	    (setq uim-preedit-cursor (point)))
+	    (if buffer-frozen
+		(uim-freeze-buffer))
+	    )
+	)
 
-	  ;; show candidate
-	  (if candidate
-	      (progn
-		(setq uim-candidate-displayed t)
-		(if (uim-check-candidate-space)
-		    (uim-show-candidate candidate))))
-	  
-	  (set-buffer-modified-p modified)
-
-	  )
-
-      ;; no preedit/candidate
-
-      ;; restore undo-list before raw key processing
-      (when uim-buffer-undo-list-saved
-	(uim-debug "call restore undo")
-	(uim-restore-undo))
-
-      ;; no raw-key and no preedit/candidate
-      (when uim-preedit-keymap-enabled
-	(uim-debug "call disable preedit keymap")
-	(uim-disable-preedit-keymap)
-	(uim-unfreeze-buffer))
 
       (if key
-	  ;; process raw key
-	  (uim-process-keyvec key))
-      )
+	  (let (keyproc-done
+		(buffer-undo-list-saved uim-buffer-undo-list-saved))
+	    (when buffer-undo-list-saved
+	      (uim-debug "call restore undo 2")
+	      (uim-restore-undo))
+	    ;; process raw key
+	    ;;  C-o is also processed here ... orz
+	    (let ((inhibit-read-only t))
+	      (unwind-protect
+		  (progn
+		    (uim-process-keyvec key)
+		    (setq keyproc-done t))
+		(when (not keyproc-done)
+		  (uim-leave-preedit-mode)
+		  (uim-debug "leave preedit mode forcibly")
+		  )
+		)
+	      ;; following expressions will not be evaluated
+	      ;; when an error occurs in this function...
+	      )
+	    (if (not uim-mode)
+		(throw 'process-agent-output t))
+	    ;; save undo hisotry again
+	    (when buffer-undo-list-saved
+	      (uim-debug "call save undo 2")
+	      (uim-save-undo))))
 
-    ;; scroll buffer after candidate removed
-    ;;  recenter if buffer is force scrolled at candidate displaying
-    (when (and candidate-existed
-	       (not uim-candidate-displayed) 
-	       uim-window-force-scrolled) 
-      (uim-debug "recenter: window-force-scrolled is true")
-      (setq uim-window-force-scrolled nil)
-      (recenter))
+      (setq uim-original-cursor (uim-point))
 
-    ;; move cursor
-    (when (or uim-preedit-displayed uim-candidate-displayed)
-      (goto-char uim-preedit-cursor))
 
-    (if (not uim-send-recv-again)
-	(when label 
-	  (setq uim-send-recv-again t)
-	  (uim-update-im-label))
-      (setq uim-send-recv-again nil))
+      (if (or preedit candidate)
+	  ;; process preedit/candidate
+	  (let ((modified (buffer-modified-p)))
 
-    ))
+	    ;; save undo list if not saved
+	    (when (not uim-buffer-undo-list-saved)
+	      (uim-flush-concat-undo)
+	      (uim-debug "call save undo 3")
+	      (uim-save-undo))
 
+	    ;; change keymap and freeze faces at first time
+	    (when (not uim-preedit-keymap-enabled)
+	      (uim-enter-preedit-mode))
+
+	    (setq uim-preedit-start uim-original-cursor)
+	    (setq uim-preedit-overlap 0)
+	    (setq uim-preedit-current-sentence-start uim-preedit-start)
+	    (setq uim-preedit-cursor uim-preedit-start)
+
+	    ;; show preedit
+	    (when preedit
+	      (setq uim-preedit-displayed t)
+	      (let ((inhibit-read-only t))
+		(uim-insert-preedit preedit)
+		))
+
+	    (uim-goto-char uim-preedit-cursor)
+
+	    (setq uim-candidate-cursor (uim-point))
+
+	    ;; show candidate
+	    (when (and candidate
+		       (uim-check-candidate-space)
+		       (eq uim-focused-buffer (current-buffer)))
+	      (setq uim-candidate-displayed t)
+	      (setq uim-candidate-start uim-preedit-current-sentence-start)
+	      (setq uim-candidate-vofs uim-preedit-overlap)
+	      (let ((inhibit-read-only t))
+		(uim-show-candidate candidate)))
+	  
+	    (uim-goto-char uim-candidate-cursor)
+
+	    (set-buffer-modified-p modified)
+	    )
+
+	;; no preedit/candidate
+
+	;; restore undo-list if saved
+	(when uim-buffer-undo-list-saved
+	  (uim-debug "call restore undo 3")
+	  (uim-restore-undo))
+
+	;; no raw-key and no preedit/candidate
+	(when uim-preedit-keymap-enabled
+	  (uim-leave-preedit-mode))
+
+	)
+
+      ;; scroll buffer after candidate removed
+      ;;  recenter if buffer is force scrolled at candidate displaying
+      (when (and candidate-existed
+		 (not uim-candidate-displayed) 
+		 uim-window-force-scrolled) 
+	(uim-debug "recenter: window-force-scrolled is true")
+	(setq uim-window-force-scrolled nil)
+	(recenter))
+
+
+      (if (not uim-send-recv-again)
+	  (when label 
+	    (setq uim-send-recv-again t)
+	    (uim-update-im-label))
+	(setq uim-send-recv-again nil))
+
+      (uim-debug (format "uim-original-cursor %s" uim-original-cursor))
+      (uim-debug "process-agent-output done")
+      ))
+  )
 
 
 
@@ -1193,23 +1372,20 @@
 (defun uim-get-im-list ()
   (uim-do-send-recv-cmd (format "0 LIST")))
 
+(defun uim-init-im-alist ()
+  (uim-get-im-list))
 
 ;;
 ;; Initialize IM list and encoding
 ;;
 (defun uim-init-im-encoding ()
 
-  (uim-get-im-list)
-
   ;; set Uim side encoding to agent
-  (let ((im-list (cdr uim-im-list)))
-    (while im-list
-      ;; cdar exists when im has valid encoding
-      (if (cdar im-list)
-	  (uim-set-encoding (caar im-list)))
-      (setq im-list (cdr im-list)))))
-
-
+  (mapcar 
+   '(lambda (x)
+      (let ((im (car x)))
+	(uim-set-encoding im (uim-get-uim-encoding im))))
+   uim-im-alist))
 
 ;;
 ;; Called when entering into the mini-buffer
@@ -1237,9 +1413,6 @@
   ;; initialize minor-mode
   (uim-init-minor-mode)
 
-  ;; initialize encoding
-  (uim-init-im-encoding)
-  
   ;; initialize keymap
   (uim-init-keymap)
 
@@ -1253,11 +1426,17 @@
   (add-hook 'minibuffer-setup-hook 'uim-minibuffer-enter)
   (add-hook 'minibuffer-exit-hook 'uim-minibuffer-exit)
 
-  (add-hook 'uim-reset-keymap-hook 'uim-update-keybind)
-
+  ;; advice to support kbd macro
+  (defadvice call-last-kbd-macro (around uim-call-last-kbd-macro activate)
+    (let ((current-uim-mode uim-mode))
+      (setq uim-mode nil)
+      (unwind-protect
+	  ad-do-it
+	(setq uim-mode current-uim-mode))))
   )
 
 
+
 ;;
 ;; Toggle uim
 ;;
@@ -1318,8 +1497,9 @@ uim mode facilitates internationalized input through the uim library."
   (when uim-initialized
     (if (not im)
 	(let (alist)
-	  (setq alist (mapcar (lambda (x) (cons (car x) (car x))) 
-			      (cdr uim-im-list)))
+	  (setq alist (mapcar '(lambda (x) 
+				 (cons (car x) (car x))) 
+			      uim-im-alist))
 	  (save-window-excursion
 	    (setq im (cdr (assoc
 			   (completing-read (concat "IM Engine: ") alist nil t)
