@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -34,7 +34,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
-#ifndef DEBUG
+#if (!defined(DEBUG) && !defined(NDEBUG))
 #define NDEBUG
 #endif
 #ifdef HAVE_SYS_TYPES_H
