@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -32,7 +32,7 @@
 */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 #include <stdio.h>
 #ifdef HAVE_STDLIB_H
@@ -100,7 +100,7 @@ int main(int argc, char **argv)
     buf[len] = '\0';
     if (strcmp(buf, prev_buf) != 0) {
       puts(buf);
-      strcpy(prev_buf, buf);
+      strlcpy(prev_buf, buf, sizeof(prev_buf));
       fflush(stdout);
     }
   }
