@@ -1,6 +1,6 @@
 /*
  *  Copyright (c) 2003,2004 Masahito Omote <omote@utyuuzin.net>
- *                2005-2006 uim Project http://uim.freedesktop.org/
+ *                2005-2007 uim Project http://uim.freedesktop.org/
  *
  *  All rights reserved.
  *
@@ -113,7 +113,7 @@ category_code substantive_code[]= {
   { "#T33", "̾(,ʽ³)"           , "", 5,  3 },
   { "#T34", "̾()"                      , "", 4,  3 },
   { "#T35", "̾(촴,ʽ³)"             , ",縰,,", 3,  3 },
-  { "#T39", "̾(̤)"                  , "", 0,  3 },
+  { "#T39", "̾"                            , "", 0,  3 },
   { "#CN",  "̾"                            , "", 0, 3 },
   { "#CNS", "̾()"                    , "", 0, 3 },
   { "#JCN", "̾()"                    , "Ĺ", 0, 3 },
@@ -328,8 +328,7 @@ const char *find_desc_from_code_with_type(const char *code, int type) {
 
   do {
     for (j = 0; j < num[i]; j++) {
-      if (strcmp(code, (category[i])[j].code) == 0
-	 && ((category[i])[j].type & type) == type)
+      if ((i == type ) && !strcmp(code, (category[i])[j].code))
 	pos = strdup((category[i])[j].desc);
     }
     i++;
@@ -361,8 +360,7 @@ const char *find_code_from_desc(const char *desc, int type) {
 
     do {
       for (j = 0; j < num[i]; j++) {
-	if (strcmp(desc, (category[i])[j].desc) == 0
-	   && ((category[i])[j].type & type) == type)
+	if ((i == type) && !strcmp(desc, (category[i])[j].desc))
 	  code = strdup((category[i])[j].code);
       }
       i++;
