@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2005-2006 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2005-2007 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -46,6 +46,7 @@
 #include "uim/gettext.h"
 
 #include "../gtk/key-util-gtk.h"
+#include "pref-gtk-keytab.h"
 
 #define DEFAULT_OLIST_WINDOW_WIDTH    480
 #define DEFAULT_OLIST_WINDOW_HEIGHT   350
@@ -1396,6 +1397,7 @@ static void
 key_pref_set_value(gint ukey, gint umod)
 {
   GString *keystr;
+  const char *sym;
 
   keystr = g_string_new("");
   /*
@@ -1425,129 +1427,15 @@ key_pref_set_value(gint ukey, gint umod)
      */
     g_string_append(keystr, "space");
     break;
-  case UKey_Backspace:
-    g_string_append(keystr, "backspace");
-    break;
-  case UKey_Delete:
-    g_string_append(keystr, "delete");
-    break;
-  case UKey_Insert:
-    g_string_append(keystr, "insert");
-    break;
-  case UKey_Escape:
-    g_string_append(keystr, "escape");
-    break;
-  case UKey_Tab:
-    g_string_append(keystr, "tab");
-    break;
-  case UKey_Return:
-    g_string_append(keystr, "return");
-    break;
-  case UKey_Left:
-    g_string_append(keystr, "left");
-    break;
-  case UKey_Up:
-    g_string_append(keystr, "up");
-    break;
-  case UKey_Right:
-    g_string_append(keystr, "right");
-    break;
-  case UKey_Down:
-    g_string_append(keystr, "down");
-    break;
-  case UKey_Prior:
-    g_string_append(keystr, "prior");
-    break;
-  case UKey_Next:
-    g_string_append(keystr, "next");
-    break;
-  case UKey_Home:
-    g_string_append(keystr, "home");
-    break;
-  case UKey_End:
-    g_string_append(keystr, "end");
-    break;
-  case UKey_Multi_key:
-    g_string_append(keystr, "Multi_key");
-    break;
-  case UKey_Mode_switch:
-    g_string_append(keystr, "Mode_switch");
-    break;
-  case UKey_Kanji:
-    g_string_append(keystr, "Kanji");
-    break;
-  case UKey_Muhenkan:
-    g_string_append(keystr, "Muhenkan");
-    break;
-  case UKey_Henkan_Mode:
-    g_string_append(keystr, "Henkan_Mode");
-    break;
-  case UKey_Romaji:
-    g_string_append(keystr, "romaji");
-    break;
-  case UKey_Hiragana:
-    g_string_append(keystr, "hiragana");
-    break;
-  case UKey_Katakana:
-    g_string_append(keystr, "katakana");
-    break;
-  case UKey_Hiragana_Katakana:
-    g_string_append(keystr, "hiragana-katakana");
-    break;
-  case UKey_Zenkaku:
-    g_string_append(keystr, "zenkaku");
-    break;
-  case UKey_Hankaku:
-    g_string_append(keystr, "hankaku");
-    break;
-  case UKey_Zenkaku_Hankaku:
-    g_string_append(keystr, "zenkaku-hankaku");
-    break;
-  case UKey_Touroku:
-    g_string_append(keystr, "touroku");
-    break;
-  case UKey_Massyo:
-    g_string_append(keystr, "massyo");
-    break;
-  case UKey_Kana_Lock:
-    g_string_append(keystr, "kana-lock");
-    break;
-  case UKey_Kana_Shift:
-    g_string_append(keystr, "kana-shift");
-    break;
-  case UKey_Eisu_Shift:
-    g_string_append(keystr, "eisu-shift");
-    break;
-  case UKey_Eisu_toggle:
-    g_string_append(keystr, "eisu-toggle");
-    break;
-  case UKey_Shift_key:
-    g_string_append(keystr, "Shift_key");
-    break;
-  case UKey_Control_key:
-    g_string_append(keystr, "Control_key");
-    break;
-  case UKey_Alt_key:
-    g_string_append(keystr, "Alt_key");
-    break;
-  case UKey_Meta_key:
-    g_string_append(keystr, "Meta_key");
-    break;
-  case UKey_Super_key:
-    g_string_append(keystr, "Super_key");
-    break;
-  case UKey_Hyper_key:
-    g_string_append(keystr, "Hyper_key");
-    break;
   default:
-    if (ukey >= UKey_F1 && ukey <= UKey_F35) {
-      g_string_append_printf(keystr, "F%d", ukey - UKey_F1 + 1);
 #if 0
-    } else if (keyval >= GDK_KP_0 && keyval <= GDK_KP_9) {
+    if (keyval >= GDK_KP_0 && keyval <= GDK_KP_9) {
       g_string_append_printf(keystr, "%d", keyval - GDK_KP_0 + UKey_0);
 #endif
-    } else if (ukey < 256) {
+    if (ukey < 128) {
       g_string_append_printf(keystr, "%c", ukey);
+    } else if ((sym = uim_pref_get_keysym(ukey))) {
+      g_string_append(keystr, sym);
     } else {
       /* UKey_Other */
     }
