@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -35,6 +35,7 @@
 #  include <config.h>
 #endif
 
+#include <stdlib.h>
 #include <locale.h>
 #include "uim/gettext.h"
 #include "uim/uim.h"
@@ -53,8 +54,16 @@ embedded_cb(GtkWidget *widget, gpointer user_data)
   uim_toolbar_check_helper_connection(user_data);
   uim_helper_client_get_prop_list();
   uim_toolbar_get_im_list();
+
+  gtk_widget_show_all(user_data);
 }
 
+static void
+systray_unrealize_cb(GtkWidget *widget, gpointer user_data)
+{
+  uim_quit();
+  exit(0);
+}
 
 int
 main(int argc, char *argv[])
@@ -77,11 +86,10 @@ main(int argc, char *argv[])
 
   icon = uim_toolbar_trayicon_new();
   g_signal_connect(G_OBJECT(tray), "embedded", G_CALLBACK(embedded_cb), icon);
-
-  gtk_widget_show_all(icon);
+  g_signal_connect_after(G_OBJECT(tray), "unrealize", G_CALLBACK(systray_unrealize_cb), NULL);
 
   gtk_container_add(GTK_CONTAINER(tray), icon);
-  gtk_widget_show(GTK_WIDGET (tray));
+  gtk_widget_show(GTK_WIDGET(tray));
 
   gtk_main();
 
