@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
 
 All rights reserved.
 
@@ -30,10 +30,15 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.
 
 */
-#include "immodule-quiminputcontext_with_slave.h"
+#include <config.h>
 
 #include <qinputcontextfactory.h>
 
+#include "uim/uim.h"
+
+#include "immodule-quiminputcontext.h"
+#include "immodule-quiminputcontext_with_slave.h"
+
 QUimInputContextWithSlave::QUimInputContextWithSlave( const char *imname, const char *lang )
         : QUimInputContext( imname, lang )
 {
@@ -50,7 +55,8 @@ QUimInputContextWithSlave::QUimInputContextWithSlave( const char *imname, const
 }
 
 QUimInputContextWithSlave::~QUimInputContextWithSlave()
-{}
+{
+}
 
 void QUimInputContextWithSlave::setFocus()
 {
@@ -91,14 +97,6 @@ void QUimInputContextWithSlave::setHolderWidget( QWidget *w )
 
 bool QUimInputContextWithSlave::filterEvent( const QEvent *event )
 {
-#if 0
-    // when isComposing==false, event is forwarded to slave ic
-    if ( ! isComposing() && slave && slave->filterEvent( event ) )
-        return true;
-
-    // else, event is dealt with uim-qt
-    return QUimInputContext::filterEvent( event );
-#else
     if ( ! QUimInputContext::filterEvent( event ) ) {
       if ( ! isComposing() && slave && slave->filterEvent( event ) )
         return true;
@@ -107,7 +105,6 @@ bool QUimInputContextWithSlave::filterEvent( const QEvent *event )
     }
 
     return true;
-#endif
 }
 
 void QUimInputContextWithSlave::destroyInputContext()
