@@ -1,6 +1,6 @@
 /*
 
- Copyright (c) 2003,2004,2005 uim Project http://uim.freedesktop.org/
+ Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
 
  All rights reserved.
 
@@ -40,6 +40,7 @@
 #define _FU8(String) QString::fromUtf8(String)
 
 #define DEBUG_KEY_EDIT 0
+static QString unicodeKeyToSymStr( QChar c );
 
 CustomCheckBox::CustomCheckBox( struct uim_custom *c, QWidget *parent, const char *name )
     : QCheckBox( parent, name ),
@@ -163,13 +164,29 @@ CustomPathnameEdit::CustomPathnameEdit( struct uim_custom *c, QWidget *parent, c
     : QHBox( parent, name ),
       UimCustomItemIface( c )
 {
+    const char *button;
+
     setSpacing( 3 );
     m_lineEdit = new QLineEdit( this );
     QObject::connect( m_lineEdit, SIGNAL(textChanged(const QString &)),
                       this, SLOT(slotCustomTextChanged(const QString &)) );
 
     m_fileButton = new QPushButton( this );
-    m_fileButton->setText( _("File") );
+    /* Since both pathname type opens the file dialog to select an item rather
+     * than open it, the label should always be "Select..." here.  The type is
+     * obvious for uses even if the button label does not indicate
+     * it. Information about the action the button causes is more important.
+     *   -- YamaKen 2006-01-21 */
+    switch (m_custom->value->as_pathname->type) {
+    case UCustomPathnameType_Directory:
+        button = N_( "Select..." );
+        break;
+    case UCustomPathnameType_RegularFile:
+    default:
+        button = N_( "Select..." );
+        break;
+    }
+    m_fileButton->setText( mygettext(button) );
     QObject::connect( m_fileButton, SIGNAL(clicked()),
                       this, SLOT(slotPathnameButtonClicked()) );
 
@@ -181,7 +198,7 @@ void CustomPathnameEdit::update()
     if( !m_custom || m_custom->type != UCustom_Pathname )
         return;
 
-    m_lineEdit->setText( _FU8(m_custom->value->as_pathname) );
+    m_lineEdit->setText( _FU8(m_custom->value->as_pathname->str) );
 
     /* sync with Label */
     parentWidget()->setEnabled( m_custom->is_active );
@@ -189,8 +206,9 @@ void CustomPathnameEdit::update()
 
 void CustomPathnameEdit::setDefault()
 {
-    free( m_custom->value->as_pathname );
-    m_custom->value->as_pathname = strdup( m_custom->default_value->as_pathname );
+    free( m_custom->value->as_pathname->str );
+    m_custom->value->as_pathname->str = strdup( m_custom->default_value->as_pathname->str );
+    m_custom->value->as_pathname->type = m_custom->default_value->as_pathname->type;
 
     setCustom( m_custom );
     update();
@@ -202,10 +220,15 @@ void CustomPathnameEdit::slotPathnameButtonClicked()
     QObject::connect( m_fileDialog, SIGNAL(filterSelected(const QString&)),
                       this, SLOT(slotFileDialogFilterSelected(const QString&)) );
 
-
-    m_fileDialog->setMode( QFileDialog::ExistingFile );
-    m_fileDialog->addFilter( "Directories" );
-    m_fileDialog->setSelectedFilter( "All Files (*)" );
+    switch (m_custom->value->as_pathname->type) {
+    case UCustomPathnameType_Directory:
+        m_fileDialog->setMode( QFileDialog::Directory );
+        break;
+    case UCustomPathnameType_RegularFile:
+    default:
+         m_fileDialog->setMode( QFileDialog::ExistingFile );
+         break;
+    }
     if ( m_fileDialog->exec() == QDialog::Accepted )
     {
         QString fileName = m_fileDialog->selectedFile();
@@ -218,8 +241,8 @@ void CustomPathnameEdit::slotCustomTextChanged( const QString & text )
 {
     Q_ASSERT( m_custom->type == UCustom_Pathname );
 
-    free( m_custom->value->as_pathname );
-    m_custom->value->as_pathname = strdup( (const char*)text.utf8() );
+    free( m_custom->value->as_pathname->str );
+    m_custom->value->as_pathname->str = strdup( (const char*)text.utf8() );
 
     setCustom( m_custom );
 }
@@ -704,8 +727,6 @@ void CustomKeyEdit::slotKeyButtonClicked()
     if( d->exec() == KeyEditForm::Accepted )
     {
         const QStringList keyStrList = d->getKeyStrList();
-        if( keyStrList.isEmpty() )
-            return;
 
         /* free old items */
         int num = 0;
@@ -847,6 +868,7 @@ KeyGrabDialog::KeyGrabDialog( QWidget *parent, const char *name )
     : QDialog( parent, name ),
       pressed_keyval( 0 ),
       pressed_keystate( Qt::NoButton ),
+      pressed_unichar ( 0 ),
       m_keystr( 0 )
 {
     QVBoxLayout *vboxLayout = new QVBoxLayout( this );
@@ -860,9 +882,10 @@ void KeyGrabDialog::keyPressEvent( QKeyEvent *e )
 {
     pressed_keyval = e->key();
     pressed_keystate = e->state();
+    pressed_unichar = e->text().at(0);
 }
 
-void KeyGrabDialog::keyReleaseEvent( QKeyEvent *e )
+void KeyGrabDialog::keyReleaseEvent( QKeyEvent * /* e */ )
 {
     // create keystr
     setKeyStr();
@@ -889,6 +912,8 @@ void KeyGrabDialog::setKeyStr()
         keystr += "<Control>";
     if( mod & Qt::AltButton )
         keystr += "<Alt>";
+    if( mod & Qt::MetaButton )
+        keystr += "<Meta>";
 
     switch( keyval ) {
     case Qt::Key_Space:
@@ -937,22 +962,162 @@ void KeyGrabDialog::setKeyStr()
         keystr += "end";
         break;
 #ifdef QT_IMMODULE
-    case Qt::Key_Kanji:
-    case Qt::Key_Zenkaku_Hankaku:
-        keystr += "zenkaku-hankaku";
-        break;
     case Qt::Key_Multi_key:
         keystr += "Multi_key";
         break;
+    case Qt::Key_Codeinput:
+        keystr += "codeinput";
+        break;
+    case Qt::Key_SingleCandidate:
+        keystr += "single-candidate";
+        break;
+    case Qt::Key_MultipleCandidate:
+        keystr += "multiple-candidate";
+        break;
+    case Qt::Key_PreviousCandidate:
+        keystr += "previous-candidate";
+        break;
     case Qt::Key_Mode_switch:
         keystr += "Mode_switch";
         break;
-    case Qt::Key_Henkan:
-        keystr += "Henkan_Mode";
+    case Qt::Key_Kanji:
+        keystr += "Kanji";
         break;
     case Qt::Key_Muhenkan:
         keystr += "Muhenkan";
         break;
+    case Qt::Key_Henkan:
+        keystr += "Henkan_Mode";
+        break;
+    case Qt::Key_Romaji:
+        keystr += "romaji";
+        break;
+    case Qt::Key_Hiragana:
+        keystr += "hiragana";
+        break;
+    case Qt::Key_Katakana:
+        keystr += "katakana";
+        break;
+    case Qt::Key_Hiragana_Katakana:
+        keystr += "hiragana-katakana";
+        break;
+    case Qt::Key_Zenkaku:
+        keystr += "zenkaku";
+        break;
+    case Qt::Key_Hankaku:
+        keystr += "hankaku";
+        break;
+    case Qt::Key_Zenkaku_Hankaku:
+        keystr += "zenkaku-hankaku";
+        break;
+    case Qt::Key_Touroku:
+        keystr += "touroku";
+        break;
+    case Qt::Key_Massyo:
+        keystr += "massyo";
+        break;
+    case Qt::Key_Kana_Lock:
+        keystr += "kana-lock";
+        break;
+    case Qt::Key_Kana_Shift:
+        keystr += "kana-shift";
+        break;
+    case Qt::Key_Eisu_Shift:
+        keystr += "eisu-shift";
+        break;
+    case Qt::Key_Eisu_toggle:
+        keystr += "eisu-toggle";
+        break;
+    case Qt::Key_Hangul:
+        keystr += "hangul";
+        break;
+    case Qt::Key_Hangul_Start:
+        keystr += "hangul-start";
+        break;
+    case Qt::Key_Hangul_End:
+        keystr += "hangul-end";
+        break;
+    case Qt::Key_Hangul_Hanja:
+        keystr += "hangul-hanja";
+        break;
+    case Qt::Key_Hangul_Jamo:
+        keystr += "hangul-jamo";
+        break;
+    case Qt::Key_Hangul_Romaja:
+        keystr += "hangul-romaja";
+        break;
+    case Qt::Key_Hangul_Jeonja:
+        keystr += "hangul-jeonja";
+        break;
+    case Qt::Key_Hangul_Banja:
+        keystr += "hangul-banja";
+        break;
+    case Qt::Key_Hangul_PreHanja:
+        keystr += "hangul-prehanja";
+        break;
+    case Qt::Key_Hangul_PostHanja:
+        keystr += "hangul-prosthanja";
+        break;
+    case Qt::Key_Hangul_Special:
+        keystr += "hangul-special";
+        break;
+    case Qt::Key_Dead_Grave:
+        keystr += "dead-grave";
+        break;
+    case Qt::Key_Dead_Acute:
+        keystr += "dead-acute";
+        break;
+    case Qt::Key_Dead_Circumflex:
+        keystr += "dead-circumflex";
+        break;
+    case Qt::Key_Dead_Tilde:
+        keystr += "dead-tilde";
+        break;
+    case Qt::Key_Dead_Macron:
+        keystr += "dead-macron";
+        break;
+    case Qt::Key_Dead_Breve:
+        keystr += "dead-breve";
+        break;
+    case Qt::Key_Dead_Abovedot:
+        keystr += "dead-abovedot";
+        break;
+    case Qt::Key_Dead_Diaeresis:
+        keystr += "dead-diaeresis";
+        break;
+    case Qt::Key_Dead_Abovering:
+        keystr += "dead-abovering";
+        break;
+    case Qt::Key_Dead_Doubleacute:
+        keystr += "dead-doubleacute";
+        break;
+    case Qt::Key_Dead_Caron:
+        keystr += "dead-caron";
+        break;
+    case Qt::Key_Dead_Cedilla:
+        keystr += "dead-cedilla";
+        break;
+    case Qt::Key_Dead_Ogonek:
+        keystr += "dead-ogonek";
+        break;
+    case Qt::Key_Dead_Iota:
+        keystr += "dead-iota";
+        break;
+    case Qt::Key_Dead_Voiced_Sound:
+        keystr += "dead-voiced-sound";
+        break;
+    case Qt::Key_Dead_Semivoiced_Sound:
+        keystr += "dead-semivoiced-sound";
+        break;
+    case Qt::Key_Dead_Belowdot:
+        keystr += "dead-belowdot";
+        break;
+    case Qt::Key_Dead_Hook:
+        keystr += "dead-hook";
+        break;
+    case Qt::Key_Dead_Horn:
+        keystr += "dead-horn";
+        break;
 #endif /* Def: QT_IMMODULE */
     case Qt::Key_Shift:
         keystr += "Shift_key";
@@ -974,6 +1139,18 @@ void KeyGrabDialog::setKeyStr()
     case Qt::Key_Hyper_R:
         keystr += "Hyper_key";
         break;
+    case Qt::Key_CapsLock:
+        keystr += "caps-lock";
+        break;
+    case Qt::Key_NumLock:
+        keystr += "num-lock";
+        break;
+    case Qt::Key_ScrollLock:
+        keystr += "scroll-lock";
+        break;
+    case Qt::Key_unknown:
+        keystr += unicodeKeyToSymStr ( pressed_unichar );
+        break;
     default:
         if( keyval >= Qt::Key_F1 && keyval <= Qt::Key_F35 )
         {
@@ -997,4 +1174,80 @@ void KeyGrabDialog::setKeyStr()
         
 }
 
+static QString unicodeKeyToSymStr ( QChar c )
+{
+    QString str = QString::null;
+
+    switch ( c.unicode() ) {
+    case 0x00A5: str = "yen"; break;
+    case 0x3002: str = "kana-fullstop"; break;
+    case 0x300C: str = "kana-opening-bracket"; break;
+    case 0x300D: str = "kana-closing-bracket"; break;
+    case 0x3001: str = "kana-comma"; break;
+    case 0x30FB: str = "kana-conjunctive"; break;
+    case 0x30F2: str = "kana-WO"; break;
+    case 0x30A1: str = "kana-a"; break;
+    case 0x30A3: str = "kana-i"; break;
+    case 0x30A5: str = "kana-u"; break;
+    case 0x30A7: str = "kana-e"; break;
+    case 0x30A9: str = "kana-o"; break;
+    case 0x30E3: str = "kana-ya"; break;
+    case 0x30E5: str = "kana-yu"; break;
+    case 0x30E7: str = "kana-yo"; break;
+    case 0x30C3: str = "kana-tsu"; break;
+    case 0x30FC: str = "kana-prolonged-sound"; break;
+    case 0x30A2: str = "kana-A"; break;
+    case 0x30A4: str = "kana-I"; break;
+    case 0x30A6: str = "kana-U"; break;
+    case 0x30A8: str = "kana-E"; break;
+    case 0x30AA: str = "kana-O"; break;
+    case 0x30AB: str = "kana-KA"; break;
+    case 0x30AD: str = "kana-KI"; break;
+    case 0x30AF: str = "kana-KU"; break;
+    case 0x30B1: str = "kana-KE"; break;
+    case 0x30B3: str = "kana-KO"; break;
+    case 0x30B5: str = "kana-SA"; break;
+    case 0x30B7: str = "kana-SHI"; break;
+    case 0x30B9: str = "kana-SU"; break;
+    case 0x30BB: str = "kana-SE"; break;
+    case 0x30BD: str = "kana-SO"; break;
+    case 0x30BF: str = "kana-TA"; break;
+    case 0x30C1: str = "kana-CHI"; break;
+    case 0x30C4: str = "kana-TSU"; break;
+    case 0x30C6: str = "kana-TE"; break;
+    case 0x30C8: str = "kana-TO"; break;
+    case 0x30CA: str = "kana-NA"; break;
+    case 0x30CB: str = "kana-NI"; break;
+    case 0x30CC: str = "kana-NU"; break;
+    case 0x30CD: str = "kana-NE"; break;
+    case 0x30CE: str = "kana-NO"; break;
+    case 0x30CF: str = "kana-HA"; break;
+    case 0x30D2: str = "kana-HI"; break;
+    case 0x30D5: str = "kana-FU"; break;
+    case 0x30D8: str = "kana-HE"; break;
+    case 0x30DB: str = "kana-HO"; break;
+    case 0x30DE: str = "kana-MA"; break;
+    case 0x30DF: str = "kana-MI"; break;
+    case 0x30E0: str = "kana-MU"; break;
+    case 0x30E1: str = "kana-ME"; break;
+    case 0x30E2: str = "kana-MO"; break;
+    case 0x30E4: str = "kana-YA"; break;
+    case 0x30E6: str = "kana-YU"; break;
+    case 0x30E8: str = "kana-YO"; break;
+    case 0x30E9: str = "kana-RA"; break;
+    case 0x30EA: str = "kana-RI"; break;
+    case 0x30EB: str = "kana-RU"; break;
+    case 0x30EC: str = "kana-RE"; break;
+    case 0x30ED: str = "kana-RO"; break;
+    case 0x30EF: str = "kana-WA"; break;
+    case 0x30F3: str = "kana-N"; break;
+    case 0x309B: str = "kana-voiced-sound"; break;
+    case 0x309C: str = "kana-semivoiced-sound"; break;
+    default:
+        break;
+    }
+
+    return str;
+}
+
 #include "pref-customwidgets.moc"
