@@ -1,6 +1,6 @@
 /*
 
- Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+ Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
 
  All rights reserved.
 
@@ -57,6 +57,9 @@
 #define N_(String) (String)
 #endif /* gettext_noop */
 
+#else /* ENABLE_NLS */
+
+#define mygettext(String) QString::fromLocal8Bit( (const char *)(String) )
 #endif /* ENABLE_NLS */
 
 #endif /* Not def: _QT_GETTEXT_H_ */
