@@ -1,6 +1,6 @@
 /*
 
- Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+ Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
 
  All rights reserved.
 
@@ -34,6 +34,7 @@
 
 #include "toolbar-standalone-qt.h"
 #include "toolbar-common-quimhelpertoolbar.h"
+#include "toolbar-common-uimstateindicator.h"
 
 #include <qapplication.h>
 #include <qpoint.h>
@@ -46,6 +47,8 @@
 #include "uim/uim.h"
 #include "qtgettext.h"
 
+#define TOOLBAR_MARGIN_SIZE	2
+
 UimStandaloneToolbar::UimStandaloneToolbar( QWidget *parent, const char *name )
     : QHBox( parent, name, Qt::WStyle_NoBorder | Qt::WX11BypassWM )
 {
@@ -55,11 +58,15 @@ UimStandaloneToolbar::UimStandaloneToolbar( QWidget *parent, const char *name )
     UimToolbarDraggingHandler *h = new UimToolbarDraggingHandler( this );
     h->adjustSize();
     h->show();
+    QObject::connect( h, SIGNAL( handleDoubleClicked() ),
+                      this, SLOT( slotToolbarDoubleClicked() ) );
+
     
-    QUimHelperToolbar *b = new QUimHelperToolbar( this );
-    b->adjustSize();
-    b->show();
-    QObject::connect( b, SIGNAL( toolbarResized() ), this, SLOT( slotToolbarResized() ) );
+    toolbar = new QUimHelperToolbar( this );
+    toolbar->adjustSize();
+    toolbar->show();
+    QObject::connect( toolbar, SIGNAL( toolbarResized() ), this, SLOT( slotToolbarResized() ) );
+    toolbar->setMargin(TOOLBAR_MARGIN_SIZE);
 
     // Move
     int panelHeight = 64; // FIXME!
@@ -73,7 +80,7 @@ UimStandaloneToolbar::UimStandaloneToolbar( QWidget *parent, const char *name )
                       this, SLOT( move( const QPoint & ) ) );
 
     // Quit
-    QObject::connect( b, SIGNAL( quitToolbar() ),
+    QObject::connect( toolbar, SIGNAL( quitToolbar() ),
                       qApp, SLOT( quit() ) );
 
     show();
@@ -89,6 +96,16 @@ UimStandaloneToolbar::slotToolbarResized()
     adjustSize();
 }
 
+void
+UimStandaloneToolbar::slotToolbarDoubleClicked()
+{
+    if (toolbar->isVisible())
+      toolbar->hide();
+    else
+      toolbar->show();
+    adjustSize();
+}
+
 UimToolbarDraggingHandler::UimToolbarDraggingHandler( QWidget *parent,
         const char* name )
         : QFrame( parent, name ),
@@ -111,8 +128,12 @@ void UimToolbarDraggingHandler::drawContents( QPainter* p )
 
 QSize UimToolbarDraggingHandler::sizeHint() const
 {
-    const int dim = style().pixelMetric( QStyle::PM_DockWindowSeparatorExtent, this );
-    return QSize( dim, 0 );
+    int width, height;
+    
+    width = style().pixelMetric( QStyle::PM_DockWindowSeparatorExtent, this );
+    height = BUTTON_SIZE + TOOLBAR_MARGIN_SIZE * 2;
+
+    return QSize( width, height );
 }
 
 QSizePolicy UimToolbarDraggingHandler::sizePolicy() const
@@ -123,17 +144,31 @@ QSizePolicy UimToolbarDraggingHandler::sizePolicy() const
 void UimToolbarDraggingHandler::mousePressEvent( QMouseEvent * /* e */ )
 {
     isDragging = true;
+    grabMouse( QCursor( Qt::SizeAllCursor) );
+
+    offsetX = QCursor::pos().x() - this->parentWidget()->x();
+    offsetY = QCursor::pos().y() - this->parentWidget()->y();
 }
 
 void UimToolbarDraggingHandler::mouseReleaseEvent( QMouseEvent * /* e */ )
 {
     isDragging = false;
+    releaseMouse();
 }
 
 void UimToolbarDraggingHandler::mouseMoveEvent( QMouseEvent * /* e */ )
 {
-    if ( isDragging )
-        emit moveTo( QCursor::pos() );
+    if ( isDragging ) {
+        QPoint pos = QCursor::pos();
+        pos -= QPoint(offsetX, offsetY);
+        emit moveTo( pos );
+    }
+}
+
+void UimToolbarDraggingHandler::mouseDoubleClickEvent( QMouseEvent * /* e */ )
+{
+    isDragging = false;
+    emit handleDoubleClicked();
 }
 
 int main( int argc, char *argv[] )
