@@ -1,7 +1,7 @@
 ;;; anthy.scm: Anthy for uim.
 ;;; charset: EUC-JP
 ;;;
-;;; Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -45,12 +45,12 @@
 (define anthy-lib-initialized? #f)
 (define anthy-version #f)
 
-(define anthy-type-direct         -1)
-(define anthy-type-hiragana        0)
-(define anthy-type-katakana        1)
-(define anthy-type-halfkana        2)
-(define anthy-type-halfwidth-alnum 3)
-(define anthy-type-fullwidth-alnum 4)
+(define anthy-type-direct          ja-type-direct)
+(define anthy-type-hiragana        ja-type-hiragana)
+(define anthy-type-katakana        ja-type-katakana)
+(define anthy-type-halfkana        ja-type-halfkana)
+(define anthy-type-halfwidth-alnum ja-type-halfwidth-alnum)
+(define anthy-type-fullwidth-alnum ja-type-fullwidth-alnum)
 
 (define anthy-input-rule-roma 0)
 (define anthy-input-rule-kana 1)
@@ -59,8 +59,11 @@
 (define anthy-candidate-type-katakana -2)
 (define anthy-candidate-type-hiragana -3)
 (define anthy-candidate-type-halfkana -4)
-(define anthy-candidate-type-halfwidth-alnum -5) ; not defined in Anthy
-(define anthy-candidate-type-fullwidth-alnum -6) ; not defined in Anthy
+;; below are not defined in Anthy
+(define anthy-candidate-type-halfwidth-alnum -5)
+(define anthy-candidate-type-fullwidth-alnum -6)
+(define anthy-candidate-type-upper-halfwidth-alnum -7)
+(define anthy-candidate-type-upper-fullwidth-alnum -8)
 
 ;; I don't think the key needs to be customizable.
 (define-key anthy-space-key? '(" "))
@@ -226,6 +229,7 @@
 		   (anthy-prepare-input-rule-activation ac)
 		   (rk-context-set-rule! (anthy-context-rkc ac)
 					 ja-rk-rule)
+		   (japanese-roma-set-yen-representation)
 		   (anthy-context-set-input-rule! ac anthy-input-rule-roma)))
 
 (register-action 'action_anthy_kana
@@ -244,6 +248,7 @@
 		   (anthy-context-set-input-rule! ac anthy-input-rule-kana)
 		   (anthy-context-change-kana-mode! ac (anthy-context-kana-mode ac))
 		   (anthy-context-set-alnum! ac #f)
+		   (japanese-roma-set-yen-representation)
 		   ;;(define-key anthy-kana-toggle-key? "")
 		   ;;(define-key anthy-on-key? generic-on-key?)
 		   ;;(define-key anthy-fullwidth-alnum-key? "")
@@ -264,6 +269,7 @@
 		   (anthy-prepare-input-rule-activation ac)
 		   (rk-context-set-rule! (anthy-context-rkc ac)
 					 ja-azik-rule)
+		   (japanese-roma-set-yen-representation)
 		   (anthy-context-set-input-rule! ac anthy-input-rule-azik)))
 
 ;; Update widget definitions based on action configurations. The
@@ -397,21 +403,36 @@
 	   (string-append-map-ustr-latter extract-kana preconv-str))))))
 
 (define anthy-make-raw-string
-  (lambda (raw-str-list wide?)
+  (lambda (raw-str-list wide? upper?)
     (if (not (null? raw-str-list))
 	(if wide?
 	    (string-append
 	     (ja-string-list-to-wide-alphabet
-	      (string-to-list (car raw-str-list)))
-	     (anthy-make-raw-string (cdr raw-str-list) wide?))
+	      (if upper?
+		  (map
+		   (lambda (x)
+		     (if (char-alphabetic? (string->charcode x))
+			 (charcode->string (char-upcase (string->charcode x)))
+			 x))
+		   (string-to-list (car raw-str-list)))
+		  (string-to-list (car raw-str-list))))
+	     (anthy-make-raw-string (cdr raw-str-list) wide? upper?))
 	    (string-append
-	     (car raw-str-list)
-	     (anthy-make-raw-string (cdr raw-str-list) wide?)))
+	     (if upper?
+		 (string-list-concat
+		  (map
+		   (lambda (x)
+		     (if (char-alphabetic? (string->charcode x))
+			 (charcode->string (char-upcase (string->charcode x)))
+			 x))
+		   (string-to-list (car raw-str-list))))
+		 (car raw-str-list))
+	     (anthy-make-raw-string (cdr raw-str-list) wide? upper?)))
 	"")))
 
 (define anthy-make-whole-raw-string
-  (lambda (ac wide?)
-    (anthy-make-raw-string (anthy-get-raw-str-seq ac) wide?)))
+  (lambda (ac wide? upper?)
+    (anthy-make-raw-string (anthy-get-raw-str-seq ac) wide? upper?)))
 
 (define anthy-init-handler
   (lambda (id im arg)
@@ -530,6 +551,35 @@
     (anthy-context-set-converting! ac #f)
     (ustr-clear! (anthy-context-segments ac))))
 
+(define kana-keys?
+  (lambda (key)
+    (if (not (symbol? key))
+	#f
+	(cond
+	 ((eq? 'kana-lock key)
+	  #f)
+	 ((eq? 'kana-shift key)
+	  #f)
+	 (else
+	   (let ((name (symbol->string key)))
+	     (if (> (string-length name) 5)
+		 (let ((keysym-head
+			(string-list-concat
+		       (list-head (reverse (string-to-list name)) 5))))
+		   (if (string=? keysym-head "-anak") ;; reverse
+		       #t
+		       #f))
+		 #f)))))))
+
+(define anthy-non-composing-symbol?
+  (lambda (ac key)
+    (if (and
+	 (symbol? key)
+	 (not (kana-keys? key))
+	 (not (eq? key 'yen)))
+	#t
+	#f)))
+
 (define anthy-proc-input-state-no-preedit
   (lambda (ac key key-state)
     (let ((rkc (anthy-context-rkc ac))
@@ -623,28 +673,41 @@
 			      anthy-type-halfwidth-alnum)))
 	    (im-commit ac (list-ref ja-space (anthy-context-kana-mode ac)))))
 
-       ((symbol? key)
+       ((anthy-non-composing-symbol? ac key)
 	(anthy-commit-raw ac))
 
        (else
 	(if (anthy-context-alnum ac)
-	    (let ((key-str (charcode->string key)))
+	    (let ((key-str (if (symbol? key)
+			       (if (symbol-bound? key)
+				   (symbol-value key)
+				   "?") ;; shouldn't happen
+			       (charcode->string key))))
 	      (ustr-insert-elem! (anthy-context-preconv-ustr ac)
 				 (if (= (anthy-context-alnum-type ac)
-				        anthy-type-halfwidth-alnum)
+					anthy-type-halfwidth-alnum)
 				     (list key-str key-str key-str)
 				     (list (ja-wide key-str) (ja-wide key-str)
-				           (ja-wide key-str))))
+					   (ja-wide key-str))))
 	      (ustr-insert-elem! (anthy-context-raw-ustr ac) key-str))
-	    (let* ((key-str (charcode->string
-			     (if (= rule anthy-input-rule-kana)
-				 key
-				 (to-lower-char key))))
+	    (let* ((key-str (if (= rule anthy-input-rule-kana)
+	    			(if (symbol? key)
+				    (symbol->string key)
+				    (charcode->string key))
+				(if (symbol? key)
+				    (symbol->string key)
+				    (charcode->string (to-lower-char key)))))
 		   (res (rk-push-key! rkc key-str)))
 	      (if res
 		  (begin
 		    (ustr-insert-elem! (anthy-context-preconv-ustr ac) res)
-		    (ustr-insert-elem! (anthy-context-raw-ustr ac) key-str))
+		    (ustr-insert-elem! (anthy-context-raw-ustr ac)
+				       (if (and (intern-key-symbol key-str)
+						(symbol-bound?
+						 (string->symbol key-str)))
+					   (symbol-value
+					    (string->symbol key-str))
+					   key-str)))
 		  (if (null? (rk-context-seq rkc))
 		      (anthy-commit-raw ac))))))))))
 
@@ -653,6 +716,20 @@
     (or (not (ustr-empty? (anthy-context-preconv-ustr ac)))
 	(> (string-length (rk-pending (anthy-context-rkc ac))) 0))))
 
+(define anthy-rotate-transposing-alnum-type
+  (lambda (cur-type state)
+    (cond
+     ((and
+       (= cur-type anthy-type-halfwidth-alnum)
+       (= state anthy-type-halfwidth-alnum))
+      anthy-candidate-type-upper-halfwidth-alnum)
+     ((and
+       (= cur-type anthy-type-fullwidth-alnum)
+       (= state anthy-type-fullwidth-alnum))
+      anthy-candidate-type-upper-fullwidth-alnum)
+     (else
+      state))))
+
 (define anthy-proc-transposing-state
   (lambda (ac key key-state)
     (let ((rotate-list '())
@@ -673,28 +750,26 @@
 	    (if (and lst
 		     (not (null? (cdr lst))))
 		(set! state (car (cdr lst)))
-		(set! state (car rotate-list))))
+		(set! state (anthy-rotate-transposing-alnum-type
+			     (anthy-context-transposing-type ac)
+			     (car rotate-list)))))
 	  (begin
 	    (anthy-context-set-transposing! ac #t)
 	    (set! state (car rotate-list))))
 
       (cond
-       ((= state anthy-type-hiragana)
-	(anthy-context-set-transposing-type! ac anthy-type-hiragana))
-       ((= state anthy-type-katakana)
-	(anthy-context-set-transposing-type! ac anthy-type-katakana))
-       ((= state anthy-type-halfkana)
-	(anthy-context-set-transposing-type! ac anthy-type-halfkana))
-       ((= state anthy-type-halfwidth-alnum)
-	(if (not (= (anthy-context-input-rule ac)
-		    anthy-input-rule-kana))
-	    (anthy-context-set-transposing-type!
-	     ac anthy-type-halfwidth-alnum)))
-       ((= state anthy-type-fullwidth-alnum)
-	(if (not (= (anthy-context-input-rule ac)
-		    anthy-input-rule-kana))
-	    (anthy-context-set-transposing-type!
-	     ac anthy-type-fullwidth-alnum)))
+       ((or
+	 (= state anthy-type-hiragana)
+	 (= state anthy-type-katakana)
+	 (= state anthy-type-halfkana))
+	(anthy-context-set-transposing-type! ac state))
+       ((or
+	 (= state anthy-type-halfwidth-alnum)
+	 (= state anthy-candidate-type-upper-halfwidth-alnum)
+	 (= state anthy-type-fullwidth-alnum)
+	 (= state anthy-candidate-type-upper-fullwidth-alnum))
+	(if (not (= (anthy-context-input-rule ac) anthy-input-rule-kana))
+	    (anthy-context-set-transposing-type! ac state)))
        (else
 	(and
 	 ; commit
@@ -733,7 +808,7 @@
 	      (anthy-prev-candidate-key? key key-state)
 	      (and (modifier-key-mask key-state)
 		   (not (shift-key-mask key-state)))
-	      (symbol? key))
+	      (anthy-non-composing-symbol? ac key))
 	     #f
 	     #t)
 	 ; implicit commit
@@ -1096,8 +1171,8 @@
 	     (not (shift-key-mask key-state)))
 	#f)
 
-       ((symbol? key)
-        #f)
+       ((anthy-non-composing-symbol? ac key)
+	#f)
 
        (else	
 	;; handle "n1" sequence as "1"
@@ -1105,10 +1180,13 @@
 		 (not (alphabet-char? key))
 		 (not (string-find
 		       (rk-expect rkc)
-		       (charcode->string
-		        (if (= rule anthy-input-rule-kana)
-			    key
-			    (to-lower-char key))))))
+		       (if (= rule anthy-input-rule-kana)
+			   (if (symbol? key)
+			       (symbol->string key)
+			       (charcode->string key))
+			   (if (symbol? key)
+			       (symbol->string key)
+			       (charcode->string (to-lower-char key)))))))
 	    (let ((pend (rk-pending rkc))
 		  (residual-kana (rk-push-key-last! rkc)))
 	      (if residual-kana
@@ -1117,7 +1195,11 @@
 		    (ustr-insert-elem! raw-str pend)))))
 
 	(if (anthy-context-alnum ac)
-	    (let ((key-str (charcode->string key))
+	    (let ((key-str (if (symbol? key)
+			       (if (symbol-bound? key)
+				   (symbol-value key)
+				   "?") ;; shouldn't happen
+			       (charcode->string key)))
 		  (pend (rk-pending rkc))
 		  (residual-kana (rk-peek-terminal-match rkc)))
 	      (rk-flush rkc) ;; OK to reset rkc here.
@@ -1132,10 +1214,13 @@
 				     (list (ja-wide key-str) (ja-wide key-str)
 					   (ja-wide key-str))))
 	      (ustr-insert-elem! raw-str key-str))
-	    (let* ((key-str (charcode->string 
-			     (if (= rule anthy-input-rule-kana)
-				 key
-				 (to-lower-char key))))
+	    (let* ((key-str (if (= rule anthy-input-rule-kana)
+	    			(if (symbol? key)
+				    (symbol->string key)
+				    (charcode->string key))
+				(if (symbol? key)
+				    (symbol->string key)
+				    (charcode->string (to-lower-char key)))))
 		   (pend (rk-pending rkc))
 		   (res (rk-push-key! rkc key-str)))
 	      (if (and res
@@ -1151,10 +1236,22 @@
 			(if (list? (car res))
 			    (begin
 			      (ustr-insert-elem! raw-str pend)
-			      (ustr-insert-elem! raw-str key-str))
+			      (ustr-insert-elem!
+			       raw-str (if (and (intern-key-symbol key-str)
+						(symbol-bound?
+						 (string->symbol key-str)))
+					   (symbol-value
+					    (string->symbol key-str))
+					   key-str)))
 			    (ustr-insert-elem!
 			     raw-str
-			     (string-append pend key-str)))))))))))))
+			     (string-append
+			      pend
+			      (if (and
+				   (intern-key-symbol key-str)
+				   (symbol-bound? (string->symbol key-str)))
+				  (symbol-value (string->symbol key-str))
+				  key-str))))))))))))))
 
 (define anthy-context-confirm-kana!
   (lambda (ac)
@@ -1227,21 +1324,19 @@
   (lambda (ac)
     (let* ((transposing-type (anthy-context-transposing-type ac)))
       (cond
-       ((= transposing-type anthy-type-hiragana)
-	(anthy-make-whole-string ac #t anthy-type-hiragana))
-
-       ((= transposing-type anthy-type-katakana)
-	(anthy-make-whole-string ac #t anthy-type-katakana))
-
-       ((= transposing-type anthy-type-halfkana)
-	(anthy-make-whole-string ac #t anthy-type-halfkana))
-
+       ((or
+	 (= transposing-type anthy-type-hiragana)
+	 (= transposing-type anthy-type-katakana)
+	 (= transposing-type anthy-type-halfkana))
+	(anthy-make-whole-string ac #t transposing-type))
        ((= transposing-type anthy-type-halfwidth-alnum)
-	(anthy-make-whole-raw-string ac #f))
-
+	(anthy-make-whole-raw-string ac #f #f))
+       ((= transposing-type anthy-candidate-type-upper-halfwidth-alnum)
+	(anthy-make-whole-raw-string ac #f #t))
        ((= transposing-type anthy-type-fullwidth-alnum)
-	(anthy-make-whole-raw-string ac #t))
-       ))))
+	(anthy-make-whole-raw-string ac #t #f))
+       ((= transposing-type anthy-candidate-type-upper-fullwidth-alnum)
+	(anthy-make-whole-raw-string ac #t #t))))))
 
 (define anthy-get-raw-str-seq
   (lambda (ac)
@@ -1273,7 +1368,15 @@
 		(if start
 		    (anthy-make-raw-string
 		     (reverse (sublist raw-str start (+ start (- len 1))))
-		     (if (= cand-idx anthy-candidate-type-halfwidth-alnum)
+		     (if (or
+			  (= cand-idx anthy-candidate-type-halfwidth-alnum)
+			  (= cand-idx
+			     anthy-candidate-type-upper-halfwidth-alnum))
+			 #f
+			 #t)
+		     (if (or
+			  (= cand-idx anthy-candidate-type-halfwidth-alnum)
+			  (= cand-idx anthy-candidate-type-fullwidth-alnum))
 			 #f
 			 #t))
 		    "??")) ;; FIXME
@@ -1473,6 +1576,20 @@
 	  (anthy-context-set-candidate-window! ac #f)))
     (anthy-context-set-candidate-op-count! ac 0)))
 
+(define anthy-rotate-segment-transposing-alnum-type
+  (lambda (idx state)
+    (cond
+     ((and
+       (= idx anthy-candidate-type-halfwidth-alnum)
+       (= state anthy-candidate-type-halfwidth-alnum))
+      anthy-candidate-type-upper-halfwidth-alnum)
+     ((and
+       (= idx anthy-candidate-type-fullwidth-alnum)
+       (= state anthy-candidate-type-fullwidth-alnum))
+      anthy-candidate-type-upper-fullwidth-alnum)
+     (else
+      state))))
+
 (define anthy-set-segment-transposing
   (lambda (ac key key-state)
     (let ((segments (anthy-context-segments ac)))
@@ -1506,12 +1623,15 @@
 		 (= idx anthy-candidate-type-katakana)
 		 (= idx anthy-candidate-type-halfkana)
 		 (= idx anthy-candidate-type-halfwidth-alnum)
-		 (= idx anthy-candidate-type-fullwidth-alnum))
+		 (= idx anthy-candidate-type-fullwidth-alnum)
+		 (= idx anthy-candidate-type-upper-halfwidth-alnum)
+		 (= idx anthy-candidate-type-upper-fullwidth-alnum))
 		(let ((lst (member idx rotate-list)))
-		  (if (and (not (null? lst))
+		  (if (and lst
 			   (not (null? (cdr lst))))
 		      (set! state (car (cdr lst)))
-		      (set! state (car rotate-list))))
+		      (set! state (anthy-rotate-segment-transposing-alnum-type
+				   idx (car rotate-list)))))
 		(set! state (car rotate-list)))
 	     (ustr-cursor-set-frontside! segments state))
 	  ;; below anthy-7802
@@ -1591,7 +1711,7 @@
 	   (not (shift-key-mask key-state)))
       #f)  ;; use #f rather than () to conform to R5RS
 
-     ((symbol? key)
+     ((anthy-non-composing-symbol? ac key)
       #f)
 
      (else
@@ -1668,4 +1788,9 @@
  anthy-get-candidate-handler
  anthy-set-candidate-index-handler
  context-prop-activate-handler
+ #f
+ #f
+ #f
+ #f
+ #f
 )
