@@ -1,6 +1,6 @@
 ;;; i18n.scm: Internationalization functions for uim
 ;;;
-;;; Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -37,7 +37,7 @@
 
 ;; convenience shorthand of runtime translation
 (define ugettext
-  (if (feature? 'nls)
+  (if (provided? "nls")
       (lambda (str)
 	(dgettext (gettext-package) str))
       (lambda (str)
