@@ -1,6 +1,6 @@
 ;;; lazy-load.scm: Lazy IM loading support
 ;;;
-;;; Copyright (c) 2005-2006 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2005-2007 uim Project http://uim.freedesktop.org/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -70,10 +70,14 @@
 	   #f ;; get-candidate-handler
 	   #f ;; set-candidate-index-handler
 	   #f ;; prop-activate-handler
+	   #f ;; input-string-handler
+	   #f ;; focus-in-handler
+	   #f ;; focus-out-handler
+	   #f ;; place-handler
+	   #f ;; displace-handler
 	   )
 	  (im-set-module-name! (retrieve-im name) module-name)))))
 
-;; TODO: rewrite test
 ;; side effect: invoke require-module for all installed IM modules
 (define stub-im-generate-stub-im-list
   (lambda (im-names)
