@@ -1,5 +1,5 @@
 ;;;
-;;; Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -181,7 +181,7 @@
    (prime-backspace-key?      . prime-command-fund-backspace)
    (prime-delete-key?         . prime-command-fund-delete)
    (prime-cancel-key?         . prime-command-fund-cancel)
-   (prime-commit-key?         . prime-command-child-commit)
+   (prime-commit-key?         . prime-command-child-finish)
    (prime-go-left-key?        . prime-command-fund-cursor-left)
    (prime-go-right-key?       . prime-command-fund-cursor-right)
    (prime-go-left-edge-key?   . prime-command-fund-cursor-left-edge)
@@ -437,11 +437,28 @@
 		     "ܸ"
 		     "PRIME򥪥"))
 		 (lambda (context)
-		   (= (prime-context-mode context)
-		      prime-mode-hiragana))
+		   (and
+		    (eq? (prime-context-language context) 'Japanese)
+		    (= (prime-context-mode context) prime-mode-hiragana)))
 		 (lambda (context)
+		   (prime-mode-language-set context 'Japanese)
 		   (prime-mode-set context prime-mode-hiragana)))
 
+(register-action 'action_prime_mode_english
+		 (lambda (context)
+		   '(ja_halfwidth_alnum
+		     "A"
+		     "Ѹ"
+		     "PRIME򥪥"))
+		 (lambda (context)
+		   (and
+		    (eq? (prime-context-language context) 'English)
+		    (= (prime-context-mode context) prime-mode-hiragana)))
+		 (lambda (context)
+		   (prime-mode-language-set context 'English)
+		   (prime-mode-set context prime-mode-hiragana)))
+
+
 (register-action 'action_prime_mode_wide_latin
 		 (lambda (context)
 		   '(ja_fullwidth_alnum
@@ -1761,13 +1778,16 @@
 
 (define prime-update-key-press
   (lambda (context)
-    (let ((session (prime-context-session context)))
+    (let ((session (prime-context-session context))
+	  (mode (prime-context-mode context)))
       (cond
        ((null? session)
 	#f)  ;; Do nothing.
 
        (else
-	(if (not (= (prime-context-mode context) prime-mode-latin))
+	(if (and
+	     (not (= mode prime-mode-latin))
+	     (not (= mode prime-mode-application)))
 	    ;; Store the current preedition into the context
 	    (prime-context-set-preedit-line!
 	     context
@@ -2028,7 +2048,7 @@
 (define prime-reset-handler
   (lambda (context)
     ;(print "prime-reset-handler")
-    ))
+    #f))
 
 (define prime-mode-set
   (lambda (context mode)
@@ -2130,4 +2150,9 @@
  prime-get-candidate-handler        ;; get-candidate
  prime-set-candidate-index-handler  ;; set-candidate-index
  context-prop-activate-handler      ;; prop
+ #f
+ #f
+ #f
+ #f
+ #f
 ) 
