@@ -1,5 +1,5 @@
 ;;;
-;;; Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -127,22 +127,14 @@
 	       (begin
 		 (if (< 0 (string-length str))
 		     (begin
-		       (if skk-use-numeric-conversion?
-			   (skk-lib-learn-word
-			    (skk-lib-replace-numeric (skk-make-string
-			    			      (skk-context-head sc)
-						      skk-type-hiragana))
-			    (skk-context-okuri-head sc)
-			    (skk-make-string (skk-context-okuri sc)
-					     skk-type-hiragana)
-			    str)
-			   (skk-lib-learn-word
-			    (skk-make-string (skk-context-head sc)
-					     skk-type-hiragana)
-			    (skk-context-okuri-head sc)
-			    (skk-make-string (skk-context-okuri sc)
-					     skk-type-hiragana)
-			    str))
+		       (skk-lib-learn-word
+			(skk-make-string (skk-context-head sc)
+					 skk-type-hiragana)
+			(skk-context-okuri-head sc)
+			(skk-make-string (skk-context-okuri sc)
+					  skk-type-hiragana)
+			 str
+			 skk-use-numeric-conversion?)
 		       (skk-save-personal-dictionary)
 		       (if skk-use-numeric-conversion?
 			  (let ((numlst
