@@ -1,6 +1,6 @@
 #!/usr/bin/env gosh
 
-;;; Copyright (c) 2005-2006 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2005-2007 uim Project http://uim.freedesktop.org/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -29,7 +29,7 @@
 ;;; SUCH DAMAGE.
 ;;;;
 
-;; This file is tested with revision 327 of new repository
+;; These tests are passed at revision 4331 (new repository)
 
 (use test.unit)
 
@@ -38,6 +38,10 @@
 (define-uim-test-case "testcase module"
   (setup
    (lambda ()
+     ;; Cancels LIBUIM_VANILLA=1. See init.scm for further details.
+     (uim '(load-enabled-modules))
+
+     (uim-define-siod-compatible-require)
      (uim '(begin
 	     ))))
 
@@ -63,10 +67,12 @@
 		 (uim '(im-module-name (retrieve-im 'hangul2))))
    ;; raw require does not set im-module-name
    (uim '(set! im-list ()))
+
    (uim '(undefine *tcode.scm-loaded*))
    (assert-false (uim-bool '(symbol-bound? '*tcode.scm-loaded*)))
    (assert-false (uim-bool '(retrieve-im 'tcode)))
    (assert-true  (uim-bool '(require "tcode.scm")))
+
    (assert-equal 'tcode
 		 (uim '(im-name (retrieve-im 'tcode))))
    (assert-false (uim-bool '(im-module-name (retrieve-im 'tcode))))
