@@ -1,4 +1,4 @@
-;;; Copyright (c) 2004-2006 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2004-2007 uim Project http://uim.freedesktop.org/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -41,7 +41,7 @@
 (if (version<? *gaunit-version* "0.1.1")
     (error "GaUnit 0.1.1 is required"))
 
-(sys-putenv "LIBUIM_SCM_FILES" "./scm")
+(sys-putenv "LIBUIM_SCM_FILES" (string-append (sys-realpath ".") "/scm"))
 (sys-putenv "LIBUIM_VERBOSE" "2")  ;; must be 1 or 2 (2 enables backtrace)
 (sys-putenv "LIBUIM_VANILLA" "1")
 
@@ -76,7 +76,7 @@
                            #f)
                          (lambda ()
                            (read in)))))
-    (if (eq? 'ERROR: uim-sh-output)
+    (if (eq? 'Error: uim-sh-output)
 	(error (uim-sh-read-error in))
 	uim-sh-output)))
 
@@ -102,6 +102,22 @@
 (define (uim-bool sexp)
   (not (null? (uim sexp))))
 
+;; only the tricky tests require this 'require' emulation.
+(define (uim-define-siod-compatible-require)
+  (uim
+   '(begin
+      (define require
+        (lambda (filename)
+          (let* ((provided-str (string-append "*" filename "-loaded*"))
+                 (provided-sym (string->symbol provided-str)))
+            (if (not (symbol-bound? provided-sym))
+                (begin
+                  (load filename)
+                  (eval (list 'define provided-sym #t)
+                        (interaction-environment))))
+            provided-sym)))
+      #t)))
+
 (eval
  (if (version>=? *gaunit-version* "0.0.6")
    '(begin
