@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -36,11 +36,11 @@
   gettext feature has been added to libuim.  -- YamaKen 2004-01-10
 */
 
-#include "config.h"
+#include <config.h>
 
 #include <stdlib.h>
 #include <string.h>
-#ifdef ENABLE_NLS
+#if ENABLE_NLS
 #include <locale.h>
 #endif
 
@@ -95,10 +95,10 @@ intl_bind_textdomain_codeset(uim_lisp domainname, uim_lisp codeset)
   uim_lisp current_codeset;
 
   if (!uim_scm_stringp(domainname)
-      || !(uim_scm_stringp(codeset) || FALSEP(codeset)))
+      || !(uim_scm_stringp(codeset) || UIM_SCM_FALSEP(codeset)))
     return uim_scm_f();
 
-  c_codeset = (FALSEP(codeset)) ? NULL : uim_scm_refer_c_str(codeset);
+  c_codeset = (UIM_SCM_FALSEP(codeset)) ? NULL : uim_scm_refer_c_str(codeset);
   c_current_codeset = bind_textdomain_codeset(uim_scm_refer_c_str(domainname),
 					      c_codeset);
   if (c_current_codeset) {
@@ -168,7 +168,7 @@ intl_dcngettext(uim_lisp domainname, uim_lisp msgid1, uim_lisp msgid2, uim_lisp
 static void
 intl_init_locale(void)
 {
-#ifdef ENABLE_NLS
+#if ENABLE_NLS
   const char *current_locale;
 
   /* Perform setlocale() only if it maybe did not performed before. This  */
@@ -207,7 +207,7 @@ uim_init_intl_subrs(void)
   uim_scm_init_subr_4("dngettext", intl_dngettext);
   uim_scm_init_subr_5("dcngettext", intl_dcngettext);
 
-#ifdef ENABLE_NLS
+#if ENABLE_NLS
   uim_scm_provide("nls");
 #endif
 }
