@@ -1,7 +1,7 @@
 /*
   uim-helper-server.c: This file will be renamed uim-helper-messagebus.c later.
 
-  Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -31,6 +31,7 @@
   SUCH DAMAGE.
 
 */
+#include <config.h>
 
 #include <sys/types.h>
 #include <pwd.h>
@@ -46,6 +47,10 @@
 #include <unistd.h>
 #include <signal.h>
 #include <fcntl.h>
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+
 #include "uim.h"
 #include "uim-helper.h"
 
@@ -94,7 +99,7 @@ init_server_fd(char *path)
 
   bzero(&myhost, sizeof(myhost));
   myhost.sun_family = PF_UNIX;
-  strcpy(myhost.sun_path, path);
+  strlcpy(myhost.sun_path, path, sizeof(myhost.sun_path));
 
   foo = bind(fd, (struct sockaddr *)&myhost, SUN_LEN(&myhost));
   if (foo < -1) {
