@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -76,7 +76,8 @@ struct uim_context_ {
   int is_enable;
 
   struct uim_code_converter *conv_if;
-  void *conv;
+  void *outbound_conv;
+  void *inbound_conv;
   char *current_im_name;
   char *short_desc;
   char *encoding;
@@ -109,9 +110,9 @@ struct uim_context_ {
   void (*candidate_selector_select_cb)(void *ptr, int index);
   void (*candidate_selector_shift_page_cb)(void *ptr, int direction);
   void (*candidate_selector_deactivate_cb)(void *ptr);
-  /* surrounding text */
-  void (*request_surrounding_text_cb)(void *ptr);
-  int (*delete_surrounding_text_cb)(void *ptr, int offset, int len);
+  /* text acquisition */
+  int (*acquire_text_cb)(void *ptr, enum UTextArea text_id, enum UTextOrigin origin, int former_len, int latter_len, char **former, char **latter);
+  int (*delete_text_cb)(void *ptr, enum UTextArea text_id, enum UTextOrigin origin, int former_len, int latter_len);
   /* configuration changed */
   void (*configuration_changed_cb)(void *ptr);
   /* switch IM */
@@ -122,6 +123,18 @@ struct uim_context_ {
   int nr_psegs;
 };
 
+
+#if 0
+/*
+  Evaluating a S-expression in C involves the two problems, performance and
+  sourcecode-simpleness. Traditional UIM_EVAL_FSTRINGn() satisfies the latter,
+  but loses former. Manual sexp construction and evaluation by a sequence of
+  function calling is an opponent. The two should co-exist until better
+  solution has been implemented as a uim-scm API.  -- YamaKen 2005-07-30
+ */
+#define UIM_EVAL_SEXP_AS_STRING
+#endif
+
 #ifdef ENABLE_NLS
 #define UIM_PREPARE_SAVING_TEXTDOMAIN_CODESET() \
     char *enc, *orig_encoding = NULL; \
@@ -271,6 +284,7 @@ extern struct uim_im *uim_im_array;
 extern int uim_nr_im;
 extern char *uim_last_client_encoding;
 
+void uim_internal_escape_string(char *str);
 #ifdef __cplusplus
 }
 #endif
