@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -59,18 +59,39 @@ extern "C" {
 #endif
 
 
+#define UIM_SCM_GCC4_READY_GC 1
+
 /*
   uim companion tools should treat lisp object as opaque. struct
   uim_opaque exists only for type check and has no actual definition.
 */
 typedef struct uim_opaque * uim_lisp;
 typedef void (*uim_func_ptr)(void);
+typedef void *(*uim_gc_gate_func_ptr)(void *);
 
+#define UIM_SCM_FALSEP(x)  (uim_scm_eq((x), uim_scm_f()))
+#define UIM_SCM_NFALSEP(x) (!uim_scm_eq((x), uim_scm_f()))
+
+#if 1
+/* deprecated: replace with UIM_SCM_N?FALSEP() */
+#define FALSEP(x)  (UIM_SCM_FALSEP(x))
+#define NFALSEP(x) (UIM_SCM_NFALSEP(x))
 #define TRUEP(x) (uim_scm_eq(x, uim_scm_t()))
-#define FALSEP(x) (uim_scm_eq(x, uim_scm_f()))
- 
 #define NTRUEP(x) (!uim_scm_eq(x, uim_scm_t()))
-#define NFALSEP(x) (!uim_scm_eq(x, uim_scm_f()))
+#endif
+
+
+#if UIM_SCM_GCC4_READY_GC
+/*
+ * The variable to ensure that a call of uim_scm_gc_protect_stack() is
+ * uninlined in portable way through (*f)().
+ *
+ * Don't access this variables directly. Use
+ * UIM_SCM_GC_PROTECTED_CALL*() instead.
+ */
+extern uim_lisp *(*volatile uim_scm_gc_current_stack_ptr)(void);
+extern uim_lisp *(*volatile uim_scm_gc_protect_stack_ptr)(uim_lisp *);
+#endif /* UIM_SCM_GCC4_READY_GC */
 
 
 /* 'uim_scm' prefix is not appropriate for these functions... any ideas? */
@@ -88,12 +109,21 @@ uim_scm_set_verbose_level(long new_value);
 void
 uim_scm_set_lib_path(const char *path);
 
+#if UIM_SCM_GCC4_READY_GC
+void
+uim_scm_gc_protect(uim_lisp *location);
+void
+uim_scm_gc_unprotect(uim_lisp *location);
+void *
+uim_scm_call_with_gc_ready_stack(uim_gc_gate_func_ptr func, void *arg);
+#else /* UIM_SCM_GCC4_READY_GC */
 void
 uim_scm_gc_protect(uim_lisp *location);
 void
 uim_scm_gc_protect_stack(uim_lisp *stack_start);
 void
 uim_scm_gc_unprotect_stack(uim_lisp *stack_start);
+#endif /* UIM_SCM_GCC4_READY_GC */
 
 /* evaluations */
 uim_bool
@@ -104,11 +134,13 @@ uim_lisp
 uim_scm_eval(uim_lisp obj);
 uim_lisp
 uim_scm_eval_c_string(const char *str);
+/* uim_scm_return_value() should only used to retrieve result of
+ * UIM_EVAL_FSTRINGn() or UIM_EVAL_STRING(). */
 uim_lisp
 uim_scm_return_value(void);
 /*
   TODO: reorganize UIM_EVAL_FSTRINGn(), uim_sizeof_sexp_str() and
-  uim_eval_string() in context.h into this file
+  uim_eval_string() in uim-internal.h into this file
 */
 
 /* type conversions */
@@ -145,18 +177,18 @@ uim_lisp
 uim_scm_make_func_ptr(uim_func_ptr func_ptr);
 
 void
-uim_scm_init_subr_0(char *name, uim_lisp (*fcn)(void));
+uim_scm_init_subr_0(const char *name, uim_lisp (*fcn)(void));
+void 
+uim_scm_init_subr_1(const char *name, uim_lisp (*fcn)(uim_lisp));
 void
-uim_scm_init_subr_1(char *name, uim_lisp (*fcn)(uim_lisp));
+uim_scm_init_subr_2(const char *name, uim_lisp (*fcn)(uim_lisp, uim_lisp));
 void
-uim_scm_init_subr_2(char *name, uim_lisp (*fcn)(uim_lisp, uim_lisp));
+uim_scm_init_subr_3(const char *name, uim_lisp (*fcn)(uim_lisp, uim_lisp, uim_lisp));
 void
-uim_scm_init_subr_3(char *name, uim_lisp (*fcn)(uim_lisp, uim_lisp, uim_lisp));
-void
-uim_scm_init_subr_4(char *name, uim_lisp (*fcn)(uim_lisp, uim_lisp, uim_lisp,
+uim_scm_init_subr_4(const char *name, uim_lisp (*fcn)(uim_lisp, uim_lisp, uim_lisp,
 		   				 uim_lisp));
 void
-uim_scm_init_subr_5(char *name, uim_lisp (*fcn)(uim_lisp, uim_lisp, uim_lisp,
+uim_scm_init_subr_5(const char *name, uim_lisp (*fcn)(uim_lisp, uim_lisp, uim_lisp,
 						uim_lisp, uim_lisp));
 
 /* constants */
@@ -185,21 +217,22 @@ uim_scm_string_equal(uim_lisp a, uim_lisp b);
 uim_lisp
 uim_scm_cons(uim_lisp car, uim_lisp cdr);
 uim_lisp
-uim_scm_car(uim_lisp cell);
+uim_scm_car(uim_lisp pair);
 uim_lisp
-uim_scm_cdr(uim_lisp cell);
+uim_scm_cdr(uim_lisp pair);
 uim_lisp
-uim_scm_cadr(uim_lisp cell);
+uim_scm_cadr(uim_lisp lst);
 uim_lisp
-uim_scm_caar(uim_lisp cell);
+uim_scm_caar(uim_lisp lst);
 uim_lisp
-uim_scm_cdar(uim_lisp cell);
+uim_scm_cdar(uim_lisp lst);
 uim_lisp
-uim_scm_cddr(uim_lisp cell);
+uim_scm_cddr(uim_lisp lst);
 uim_lisp
-uim_scm_length(uim_lisp list);
+uim_scm_length(uim_lisp lst);
 uim_lisp
-uim_scm_reverse(uim_lisp cell);
+uim_scm_reverse(uim_lisp lst);
+
 
 #ifdef __cplusplus
 }
