@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -31,7 +31,7 @@
 
 */
 
-#include "config.h"
+#include <config.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -39,7 +39,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <locale.h>
-#include <langinfo.h>
 #include "uim-internal.h"
 #include "gettext.h"
 #include "uim-scm.h"
@@ -244,7 +243,7 @@ rk_find_seq(uim_lisp seq, uim_lisp rules)
   for (; !uim_scm_nullp(rules); rules = uim_scm_cdr(rules)) {
     uim_lisp rule = uim_scm_car(rules);
     uim_lisp key = uim_scm_car(uim_scm_car(rule));
-    if NFALSEP(str_seq_equal(seq, key)) {
+    if UIM_SCM_NFALSEP(str_seq_equal(seq, key)) {
       return rule;
     }
   }
@@ -257,7 +256,7 @@ rk_find_partial_seq(uim_lisp seq, uim_lisp rules)
   for (; !uim_scm_nullp(rules); rules = uim_scm_cdr(rules)) {
     uim_lisp rule = uim_scm_car(rules);
     uim_lisp key = uim_scm_car(uim_scm_car(rule));
-    if NFALSEP(str_seq_partial(seq, key)) {
+    if UIM_SCM_NFALSEP(str_seq_partial(seq, key)) {
       return rule;
     }
   }
@@ -276,7 +275,7 @@ rk_expect_seq(uim_lisp seq, uim_lisp rules)
     uim_lisp rule = uim_scm_car(cur);
     uim_lisp key = uim_scm_caar(rule);
     uim_lisp e = str_seq_partial(seq, key);
-    if NFALSEP(e) {
+    if UIM_SCM_NFALSEP(e) {
       res = uim_scm_cons(e, res);
     }
   }
@@ -306,7 +305,7 @@ c_setenv(uim_lisp name_, uim_lisp val_, uim_lisp overwrite_)
 {
   const char *name = uim_scm_refer_c_str(name_);
   const char *val = uim_scm_refer_c_str(val_);
-  int overwrite = NFALSEP(overwrite_);
+  int overwrite = UIM_SCM_NFALSEP(overwrite_);
   int err;
 
   if (!name || !val) {
@@ -376,8 +375,7 @@ uim_strsplit(const char *splittee, const char *splitter)
     }
     /* store */
     res[i] = malloc(sizeof(char) * (len + 1));
-    strncpy(res[i], cur, len);
-    res[i][len] = 0;
+    strlcpy(res[i], cur, len + 1);
     cur = tmp;
   }
   /**/
@@ -446,7 +444,7 @@ eucjp_string_to_list(uim_lisp str_)
 
 static uim_lisp
 string_prefixp_internal(uim_lisp prefix_, uim_lisp str_,
-			int cmp(const char *, const char *, size_t))
+			int (*cmp)(const char *, const char *, size_t))
 {
   const char *prefix, *str;
   size_t len;
@@ -458,7 +456,7 @@ string_prefixp_internal(uim_lisp prefix_, uim_lisp str_,
   str = uim_scm_refer_c_str(str_);
   len = strlen(prefix);
 
-  return cmp(prefix, str, len) ? uim_scm_f() : uim_scm_t();
+  return (*cmp)(prefix, str, len) ? uim_scm_f() : uim_scm_t();
 }
 
 static uim_lisp
@@ -502,7 +500,9 @@ iterate_lists(uim_lisp mapper, uim_lisp seed, uim_lisp lists)
   uim_bool single_listp;
 
   single_listp = uim_scm_nullp(uim_scm_cdr(lists));
+  rest = rests = uim_scm_null_list();
   res = seed;
+
   if (single_listp) {
     rest = uim_scm_car(lists);
   } else {
@@ -519,7 +519,7 @@ iterate_lists(uim_lisp mapper, uim_lisp seed, uim_lisp lists)
       }
     } else {
       pair = shift_elems(rests);
-      if (FALSEP(pair)) {
+      if (UIM_SCM_FALSEP(pair)) {
 	elms = rests = uim_scm_null_list();
       } else {
 	elms = uim_scm_car(pair);
@@ -533,7 +533,7 @@ iterate_lists(uim_lisp mapper, uim_lisp seed, uim_lisp lists)
     mapped = uim_scm_eval(form);
     termp = uim_scm_car(mapped);
     res = uim_scm_cdr(mapped);
-  } while (FALSEP(termp));
+  } while (UIM_SCM_FALSEP(termp));
 
   return res;
 }
@@ -546,7 +546,7 @@ find_tail(uim_lisp pred, uim_lisp lst)
   for (; !uim_scm_nullp(lst); lst = uim_scm_cdr(lst)) {
     elem = uim_scm_car(lst);
     form = uim_scm_list2(pred, uim_scm_quote(elem));
-    if (NFALSEP(uim_scm_eval(form)))
+    if (UIM_SCM_NFALSEP(uim_scm_eval(form)))
       return lst;
   }
 
@@ -615,9 +615,9 @@ lang_code_to_lang_name_raw(uim_lisp code_)
 }
 
 static uim_lisp
-is_setugidp(void)
+setugidp(void)
 {
-  if (is_setugid()) {
+  if (uim_issetugid()) {
     return uim_scm_t();
   }
   return uim_scm_f();
@@ -655,5 +655,5 @@ uim_init_util_subrs(void)
   uim_scm_init_subr_3("iterate-lists", iterate_lists);
   uim_scm_init_subr_2("find-tail", find_tail);
   uim_scm_init_subr_1("lang-code->lang-name-raw", lang_code_to_lang_name_raw);
-  uim_scm_init_subr_0("is-set-ugid?", is_setugidp);
+  uim_scm_init_subr_0("setugid?", setugidp);
 }
