@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -135,7 +135,8 @@ uim_create_context(void *ptr,
   uc->short_desc = NULL;
   uc->encoding = strdup(enc);
   uc->conv_if = conv;
-  uc->conv = NULL;
+  uc->outbound_conv = NULL;
+  uc->inbound_conv = NULL;
   /**/
   uc->nr_modes = 0;
   uc->modes = NULL;
@@ -158,8 +159,8 @@ uim_create_context(void *ptr,
   uc->candidate_selector_shift_page_cb = NULL;
   uc->candidate_selector_deactivate_cb = NULL;
   /**/
-  uc->request_surrounding_text_cb = NULL;
-  uc->delete_surrounding_text_cb = NULL;
+  uc->acquire_text_cb = NULL;
+  uc->delete_text_cb = NULL;
   /**/
   uc->configuration_changed_cb = NULL;
   /**/
@@ -171,7 +172,7 @@ uim_create_context(void *ptr,
   /**/
   uc->psegs = NULL;
   uc->nr_psegs = 0;
-
+  
   if (!lang) {
     lang = "#f";
   }
@@ -208,6 +209,30 @@ uim_reset_context(uim_context uc)
    uim_release_preedit_segments(uc);
 }
 
+void
+uim_focus_in_context(uim_context uc)
+{
+   UIM_EVAL_FSTRING1(uc, "(focus-in-handler %d)", uc->id);
+}
+
+void
+uim_focus_out_context(uim_context uc)
+{
+   UIM_EVAL_FSTRING1(uc, "(focus-out-handler %d)", uc->id);
+}
+
+void
+uim_place_context(uim_context uc)
+{
+   UIM_EVAL_FSTRING1(uc, "(place-handler %d)", uc->id);
+}
+
+void
+uim_displace_context(uim_context uc)
+{
+   UIM_EVAL_FSTRING1(uc, "(displace-handler %d)", uc->id);
+}
+
 void
 uim_set_configuration_changed_cb(uim_context uc,
 				 void (*changed_cb)(void *ptr))
@@ -271,9 +296,10 @@ uim_release_context(uim_context uc)
 
   UIM_EVAL_FSTRING1(uc, "(release-context %d)", uc->id);
   put_context_id(uc);
-  if (uc->conv) {
-    uc->conv_if->release(uc->conv);
-  }
+  if (uc->outbound_conv)
+    uc->conv_if->release(uc->outbound_conv);
+  if (uc->inbound_conv)
+    uc->conv_if->release(uc->inbound_conv);
   uim_release_preedit_segments(uc);
   for (i = 0; i < uc->nr_modes; i++) {
     free(uc->modes[i]);
@@ -561,15 +587,15 @@ uim_get_candidate(uim_context uc, int index, int accel_enumeration_hint)
 		    uc->id, index, accel_enumeration_hint);
 
   if (uim_return_str_list[0] && uim_return_str_list[1]) {
-    cand->str = uc->conv_if->convert(uc->conv, uim_return_str_list[0]);
-    cand->heading_label = uc->conv_if->convert(uc->conv, uim_return_str_list[1]);    
+    cand->str = uc->conv_if->convert(uc->outbound_conv, uim_return_str_list[0]);
+    cand->heading_label = uc->conv_if->convert(uc->outbound_conv, uim_return_str_list[1]);    
   } else {
     cand->str = NULL;
     cand->heading_label = NULL;
   }
 
   if (uim_return_str_list[2]) {
-    cand->annotation = uc->conv_if->convert(uc->conv, uim_return_str_list[2]);
+    cand->annotation = uc->conv_if->convert(uc->outbound_conv, uim_return_str_list[2]);
   }
 
   return cand;
@@ -598,9 +624,7 @@ uim_candidate_free(uim_candidate cand)
 {
   free(cand->str);
   free(cand->heading_label);
-  if (cand->annotation)
-    free(cand->annotation);
-
+  free(cand->annotation);
   free(cand);
 }
 
@@ -617,18 +641,70 @@ uim_set_candidate_index(uim_context uc, int nth)
 }
 
 void
-uim_set_surrounding_text_cb(uim_context uc,
-			    void (*request_cb)(void *ptr),
-			    int (*delete_cb)(void *ptr, int offset, int len))
+uim_set_text_acquisition_cb(uim_context uc,
+			    int (*acquire_cb)(void *ptr,
+					      enum UTextArea text_id,
+					      enum UTextOrigin origin,
+					      int former_len, int latter_len,
+					      char **former, char **latter),
+			    int (*delete_cb)(void *ptr, enum UTextArea text_id,
+				    	     enum UTextOrigin origin,
+					     int former_len, int latter_len))
 {
-  uc->request_surrounding_text_cb = request_cb;
-  uc->delete_surrounding_text_cb = delete_cb;
+  uc->acquire_text_cb = acquire_cb;
+  uc->delete_text_cb = delete_cb;
 }
 
 void
-uim_set_surrounding_text(uim_context uc, const char *text,
-			 int cursor_pos, int len)
+uim_internal_escape_string(char *str)
 {
+  char *p;
+  int len;
+  
+  if (!str)
+    return;
+
+  len = strlen(str);
+
+  for (p = str; *p != '\0'; p++) {
+    switch (*p) {
+    case '"':
+    case '\\':
+      str = realloc(str, len + 2);
+      if (!str)
+        return;
+      memmove(p + 1, p, str + len - p + 1);
+      *p = '\\';
+      len++;
+      p++;
+      break;
+    default:
+      break;
+    }
+  }
+}
+
+uim_bool
+uim_input_string(uim_context uc, const char *str)
+{
+  uim_lisp consumed;
+  char *conv;
+
+  conv = uc->conv_if->convert(uc->inbound_conv, str);
+  if (conv) {
+    uim_internal_escape_string(conv);
+    if (conv) {
+      UIM_EVAL_FSTRING2(uc, "(input-string-handler %d \"%s\")", uc->id, conv);
+      consumed = uim_scm_return_value();
+    } else {
+      consumed = uim_scm_f();
+    }
+    free(conv);
+
+    return uim_scm_c_bool(consumed);
+  }
+
+  return UIM_FALSE;
 }
 
 static void
