@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -41,7 +41,7 @@ public:
     Canddisp();
     ~Canddisp();
     void activate(std::vector<const char *>, int display_limit);
-    void select(int index);
+    void select(int index, bool need_hilite);
     void deactivate();
     void show();
     void hide();
