@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -33,7 +33,7 @@
 // connection management
 
 #ifdef HAVE_CONFIG_H
-# include "config.h"
+# include <config.h>
 #endif
 
 #include <unistd.h>
@@ -48,6 +48,8 @@
 # include <alloca.h>
 #endif
 
+#include "uim/uim-stdint.h"
+
 #define BUF_SIZE 1024
 #define TRANSPORT_UNIT 20
 #define TRANSPORT_MAX 20	// Emacs's XIM won't work correctly when the value is 100
@@ -93,7 +95,7 @@ static void
 reapOldConnection()
 {
     std::map<Window, XConnection *>::iterator it;
-    for (it = gXConnections.begin(); it != gXConnections.end(); it++) {
+    for (it = gXConnections.begin(); it != gXConnections.end(); ++it) {
 	XConnection *xc = (*it).second;
 	if (!xc->isValid()) {
 	    delete xc;
@@ -150,7 +152,7 @@ XConnection::~XConnection()
     free(mBuf.buf);
 }
 
-void XConnection::destroy(Window w)
+void XConnection::destroy(Window /* w */)
 {
     if (mIsValid)
 	OnClose();
@@ -227,7 +229,7 @@ bool XConnection::readToBuf(XClientMessageEvent *ev)
 			       offset, BUF_SIZE - mBuf.len, True,
 			       AnyPropertyType,
 			       &type, &format, &nrItems, &remain,
-			       (unsigned char **)&data);
+			       (unsigned char **)(uintptr_t)&data);
 	    if (!data)
 		return false;
 
@@ -256,7 +258,7 @@ void XConnection::writePendingPacket()
     bool sent_preedit_done = false;
     int major;
 
-    while (mPendingTxQ.size()) {
+    while (!mPendingTxQ.empty()) {
 	if (hasSyncFlag() || hasPreeditStartSyncFlag() ||
 			hasPreeditCaretSyncFlag())
 	    break;
@@ -291,7 +293,7 @@ void XConnection::writePendingPacket()
 	    // push first XIM_COMMIT into the head of the queue
 	    std::list<TxPacket *> tmp;
 	    bool first = true;
-	    while (mPendingTxQ.size()) {
+	    while (!mPendingTxQ.empty()) {
 		j = mPendingTxQ.begin();
 		major = (*j)->get_major();
 		if (major == XIM_COMMIT && first == true) {
@@ -314,8 +316,8 @@ void XConnection::writePassivePacket()
     bool sent_preedit_done = false;
     int major;
 
-    while (mPTxQ.size()) {
-	if (mPendingTxQ.size())
+    while (!mPTxQ.empty()) {
+	if (!mPendingTxQ.empty())
 	    break;
 
 	i = mPTxQ.begin();
@@ -355,7 +357,7 @@ void XConnection::writePassivePacket()
 	    // push first XIM_COMMIT into the head of passive queue
 	    std::list<TxPacket *> tmp;
 	    bool first = true;
-	    while (mPTxQ.size()) {
+	    while (!mPTxQ.empty()) {
 		j = mPTxQ.begin();
 		major = (*j)->get_major();
 		if (major == XIM_COMMIT && first == true) {
@@ -377,7 +379,7 @@ void XConnection::writeNormalPacket()
     std::list<TxPacket *>::iterator i;
     int major;
 
-    while (mTxQ.size()) {
+    while (!mTxQ.empty()) {
 	i = mTxQ.begin();
 	major = (*i)->get_major();
 	if (major == XIM_FORWARD_EVENT) {
