@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -33,7 +33,7 @@
 // Handle IM procedure defined in XIM protocol
 
 #ifdef HAVE_CONFIG_H
-# include "config.h"
+# include <config.h>
 #endif
 
 #include <string.h>
@@ -154,7 +154,7 @@ void XimIM_impl::destroy_ic(int icid)
 void XimIM_impl::changeContext(const char *engine)
 {
     std::map<int, XimIC *>::iterator i;
-    for (i = m_ics.begin(); i != m_ics.end(); i++) {
+    for (i = m_ics.begin(); i != m_ics.end(); ++i) {
 	(*i).second->changeContext(engine);
     }
     if (mEngineName)
@@ -234,7 +234,7 @@ int XimIM_impl::unused_ic_id()
 {
     std::map<int, XimIC *>::iterator i;
     int max_id = 1; // Does ID of input-context start with 1?
-    for (i = m_ics.begin(); i != m_ics.end(); i++) {
+    for (i = m_ics.begin(); i != m_ics.end(); ++i) {
 	if (max_id <= (*i).first)
 	    max_id = (*i).first + 1;
     }
@@ -322,13 +322,19 @@ void XimIM_impl::forward_event(RxPacket *p)
 	k.press = (k.ev.type == KeyPress);
 	k.key_sym = ks;
 
-	if (ic)
+	if (ic) {
+	    InputContext *focusedContext = InputContext::focusedContext();
+	    if (!focusedContext)
+		ic->setFocus(); // workaround for some buggy applications
 	    ic->OnKeyEvent(k);
+	}
 	if (!(g_option_mask & OPT_ON_DEMAND_SYNC))
 	    send_sync_reply(icid);
 	break;
     default:
 	printf("unknown type of forwarded event.(%d)\n", k.ev.type);
+	if (!(g_option_mask & OPT_ON_DEMAND_SYNC))
+	    send_sync_reply(icid);
 	break;
     }
 }
@@ -336,7 +342,7 @@ void XimIM_impl::forward_event(RxPacket *p)
 void XimIM_impl::free_all_ic()
 {
     std::map<int, XimIC *>::iterator i;
-    for (i = m_ics.begin(); i != m_ics.end(); i++) {
+    for (i = m_ics.begin(); i != m_ics.end(); ++i) {
 	(*i).second->unsetFocus();
 	delete (*i).second;
     }
@@ -346,7 +352,7 @@ void XimIM_impl::free_all_ic()
 void XimIM_impl::delete_ic(XimIC *ic)
 {
     std::map<int, XimIC *>::iterator it;
-    for (it = m_ics.begin(); it != m_ics.end(); it++) {
+    for (it = m_ics.begin(); it != m_ics.end(); ++it) {
 	if (it->second == ic) {
 	    it->second->unsetFocus();
 	    delete it->second;
@@ -375,7 +381,7 @@ void XimIM_impl::send_sync(int icid)
 void XimIM_impl::onSendPacket()
 {
     std::map<int, XimIC *>::iterator i;
-    for (i = m_ics.begin(); i != m_ics.end(); i++) {
+    for (i = m_ics.begin(); i != m_ics.end(); ++i) {
 	(*i).second->onSendPacket();
     }
 }
@@ -420,8 +426,22 @@ void XimIM::set_encoding(const char *encoding)
 
     // set iconv environment
     if (mLocale)
-      delete mLocale;
-    mLocale = createLocale(mEncoding);
+	delete mLocale;
+    // workaround for Solaris 10 (bug #7558)
+    char *p;
+    if (!strcasecmp(encoding, "EUC") && mLangRegion &&
+	(p = strchr(mLangRegion, '_'))) {
+	char *iconv_encoding = (char *)malloc(3 + strlen(p + 1) + 1);
+	if (iconv_encoding) {
+	    sprintf(iconv_encoding, "euc%s", p + 1);
+	    mLocale = createLocale(iconv_encoding);
+	    free(iconv_encoding);
+	} else {
+	    mLocale = createLocale(mEncoding);
+	}
+    } else {
+	mLocale = createLocale(mEncoding);
+    }
 }
 
 const char *XimIM::get_encoding()
@@ -472,7 +492,7 @@ int unused_im_id()
     int max_id;
     std::map<int, XimIM *>::iterator i;
     max_id = 1;
-    for (i = g_ims.begin(); i != g_ims.end(); i++) {
+    for (i = g_ims.begin(); i != g_ims.end(); ++i) {
 	if ((*i).first == max_id)
 	    max_id = (*i).first + 1;
     }
