@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -40,7 +40,18 @@
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #include <X11/Xutil.h>
+
+/* workaround for pre X11R6.7 */
+#ifndef XK_KOREAN
+#define XK_KOREAN
+#endif
+#ifndef XK_KATAKANA
+#define XK_KATAKANA
+#endif
 #include <X11/keysymdef.h>
+#ifndef XK_dead_horn
+#define XK_dead_horn	0xfe62
+#endif
 
 #include "xim.h"
 #include "convdisp.h"
@@ -542,11 +553,13 @@ InputContext::focusIn()
     uim_prop_label_update(mUc);	
     if (hasActiveCandwin())
 	candidate_update();
+    uim_focus_in_context(mUc);
 }
 
 void
 InputContext::focusOut()
 {
+    uim_focus_out_context(mUc);
     uim_helper_client_focus_out(mUc);
     if (mFocusedContext == this) {
 	Canddisp *disp = canddisp_singleton();
@@ -607,6 +620,7 @@ void InputContext::candidate_activate_cb(void *ptr, int nr, int display_limit)
 void InputContext::candidate_select_cb(void *ptr, int index)
 {
     InputContext *ic = (InputContext *)ptr;
+    ic->set_need_hilite_selected_cand(true);
     ic->candidate_select(index);
 }
 
@@ -812,6 +826,10 @@ void InputContext::candidate_activate(int nr, int display_limit)
     mDisplayLimit = display_limit;
     if (display_limit)
 	mNumPage = (nr - 1) / display_limit + 1;
+
+    current_cand_selection = 0;
+    current_page = 0;
+    need_hilite_selected_cand = false;
 }
 
 void InputContext::candidate_update()
@@ -819,14 +837,14 @@ void InputContext::candidate_update()
     Canddisp *disp = canddisp_singleton();
 
     disp->activate(active_candidates, mDisplayLimit);
-    disp->select(current_cand_selection);
+    disp->select(current_cand_selection, need_hilite_selected_cand);
     disp->show();
 }
 
 void InputContext::candidate_select(int index)
 {
     Canddisp *disp = canddisp_singleton();
-    disp->select(index);
+    disp->select(index, need_hilite_selected_cand);
     current_cand_selection = index;
     if (mDisplayLimit)
 	current_page = current_cand_selection / mDisplayLimit;
@@ -858,7 +876,8 @@ void InputContext::candidate_shift_page(int direction)
 	    current_cand_selection = new_index;
     }
     candidate_select(current_cand_selection);
-    uim_set_candidate_index(mUc, current_cand_selection);
+    if (need_hilite_selected_cand)
+      uim_set_candidate_index(mUc, current_cand_selection);
 }
 
 void InputContext::candidate_deactivate()
@@ -876,6 +895,11 @@ void InputContext::candidate_deactivate()
     }
 }
 
+void InputContext::set_need_hilite_selected_cand(bool set)
+{
+    need_hilite_selected_cand = set;
+}
+
 char *InputContext::get_caret_state_label_from_prop_list(const char *str)
 {
     const char *p, *q;
@@ -1076,14 +1100,24 @@ void keyState::check_key(keyEventX *x)
 	mKey = x->key_sym;
     else if (x->key_sym >= XK_F1 && x->key_sym <= XK_F35)
 	mKey = x->key_sym - XK_F1 + UKey_F1;
-    else if (x->key_sym >= 0xfe50 && x->key_sym <= 0xfe62)
-	mKey = UKey_Other; // Don't forward deadkeys to libuim.
+    // GTK+ and Qt don't support dead_stroke yet
+    else if (x->key_sym >= XK_dead_grave && x->key_sym <= XK_dead_horn)
+	mKey = x->key_sym - XK_dead_grave + UKey_Dead_Grave;
+    else if (x->key_sym >= XK_Kanji && x->key_sym <= XK_Eisu_toggle)
+	mKey = x->key_sym - XK_Kanji + UKey_Kanji;
+    else if (x->key_sym >= XK_Hangul && x->key_sym <= XK_Hangul_Special)
+	mKey = x->key_sym - XK_Hangul + UKey_Hangul;
+    else if (x->key_sym >= XK_kana_fullstop && x->key_sym <= XK_semivoicedsound)
+	mKey = x->key_sym - XK_kana_fullstop + UKey_Kana_Fullstop;
     else {
 	switch (x->key_sym) {
+	case XK_yen: mKey = UKey_Yen; break;
 	case XK_BackSpace: mKey = UKey_Backspace; break;
 	case XK_Delete: mKey = UKey_Delete; break;
+	case XK_Insert: mKey = UKey_Insert; break;
 	case XK_Escape: mKey = UKey_Escape; break;
-	case XK_Tab: mKey = UKey_Tab; break;
+	case XK_Tab:
+	case XK_ISO_Left_Tab: mKey = UKey_Tab; break;
 	case XK_Return: mKey = UKey_Return; break;
 	case XK_Left: mKey = UKey_Left; break;
 	case XK_Up: mKey = UKey_Up; break;
@@ -1094,23 +1128,11 @@ void keyState::check_key(keyEventX *x)
 	case XK_Home: mKey = UKey_Home; break;
 	case XK_End: mKey = UKey_End; break;
 	case XK_Multi_key: mKey = UKey_Multi_key; break;
+	case XK_Codeinput: mKey = UKey_Codeinput; break;
+	case XK_SingleCandidate: mKey = UKey_SingleCandidate; break;
+	case XK_MultipleCandidate: mKey = UKey_MultipleCandidate; break;
+	case XK_PreviousCandidate: mKey = UKey_PreviousCandidate; break;
 	case XK_Mode_switch: mKey = UKey_Mode_switch; break;
-	case XK_Kanji: mKey = UKey_Kanji; break;
-	case XK_Muhenkan: mKey = UKey_Muhenkan; break;
-	case XK_Henkan_Mode: mKey = UKey_Henkan_Mode; break;
-	case XK_Romaji: mKey = UKey_Romaji; break;
-	case XK_Hiragana: mKey = UKey_Hiragana; break;
-	case XK_Katakana: mKey = UKey_Katakana; break;
-	case XK_Hiragana_Katakana: mKey = UKey_Hiragana_Katakana; break;
-	case XK_Zenkaku: mKey = UKey_Zenkaku; break;
-	case XK_Hankaku: mKey = UKey_Hankaku; break;
-	case XK_Zenkaku_Hankaku: mKey = UKey_Zenkaku_Hankaku; break;
-	case XK_Touroku: mKey = UKey_Touroku; break;
-	case XK_Massyo: mKey = UKey_Massyo; break;
-	case XK_Kana_Lock: mKey = UKey_Kana_Lock; break;
-	case XK_Kana_Shift: mKey = UKey_Kana_Shift; break;
-	case XK_Eisu_Shift: mKey = UKey_Eisu_Shift; break;
-	case XK_Eisu_toggle: mKey = UKey_Eisu_toggle; break;
 	case XK_Shift_L: mKey = UKey_Shift_key; break;
 	case XK_Shift_R: mKey = UKey_Shift_key; break;
 	case XK_Control_L: mKey = UKey_Control_key; break;
@@ -1123,6 +1145,9 @@ void keyState::check_key(keyEventX *x)
 	case XK_Super_R: mKey = UKey_Super_key; break;
 	case XK_Hyper_L: mKey = UKey_Hyper_key; break;
 	case XK_Hyper_R: mKey = UKey_Hyper_key; break;
+	case XK_Caps_Lock: mKey = UKey_Caps_Lock; break;
+	case XK_Num_Lock: mKey = UKey_Num_Lock; break;
+	case XK_Scroll_Lock: mKey = UKey_Scroll_Lock; break;
 	default:
 	    mKey = UKey_Other;
 	}
